import os
import time
import textfsm

from mdc.router.cli.rp_parser.edrg9010_v2_0.parser import parse_colon_fields
# execute __init__.py for inheritance
from mdc.router.cli.rp_parser import edrg9010_v3_0, edrg9010_v2_0, oncellg4302_v3_0, tn5916_v3_0, tn5916_v3_0_27, edr810_v5_0, edr8010_v3_0
from session import PROMPTS

dirname = os.path.dirname(__file__)

def ping(session: object, host, model: str):
    """
    Execute the 'ping' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): The session object to use for the ping command.
            host (str): The host to ping.
            model (str): The model of the device.
            
        Returns: 
            dict: A dictionary containing the parsed output of the ping command. 
    """
    filename = os.path.join(
        dirname, f'{model.__name__}/moxa_ng_router_show_ping.textfsm')
    ret = session.command_expect(f'ping {host}',
                                 prompts=["Lost"],
                                 exact_str=True,
                                 timeout=30)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res[0]


def show_version(session: object, model: str):
    """
    Execute the 'show version' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): The session object to use for connecting to the device. 
            model (str): The model of the device. 
            
        Returns: 
            res (list): A list of dictionaries containing the version information. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_version.textfsm')
    ret = session.command_expect('show version',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=1)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_system(session: object, model: str):
    """
    Execute the 'show system' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): An instance of Session object.
            model (str): The model of the Moxa NG router.

        Returns:
            list: A list of dictionaries containing the parsed output from the 'show system' command. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_system.textfsm')
    ret = session.command_expect('show system',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=1)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_package(session: object, model: str):
    """
    Execute the 'show package' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): An instance of Session object.
            model (str): The model of the Moxa NG router.

        Returns:
            list: A list of dictionaries containing the parsed output from the 'show system' command. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_package.textfsm')
    ret = session.command_expect('show package',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=1)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_dos(session: object, model: str):
    """
    Execute the 'show dos' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): An instance of Session object.
            model (str): The model of the Moxa NG router.

        Returns:
            list: A list of dictionaries containing the parsed output from the 'show dos' command.
    """
    filename = os.path.join(
        dirname, f'{model.__name__}/moxa_ng_router_show_dos.textfsm')
    ret = session.command_expect('show dos',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=1)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_interface(session: object, name: str, model: str, vid=None):
    """
    Execute the 'show interface' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): An instance of the Netmiko class.
            name (str): The name of the interface to show.
            model (str): The model of the router.
            vid (int, optional): The VLAN ID associated with the interface. Defaults to None. 
            
        Returns:
            list: A list of dictionaries containing the parsed output from the router. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_interface_{name.lower()}.textfsm'
    )
    if vid == None:
        ret = session.command_expect(f'show interface {name.lower()}',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     exact_str=True,
                                     timeout=5)
    else:
        ret = session.command_expect(f'show interface {name.lower()} {vid}',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     exact_str=True,
                                     timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


# this just only router v1.2 can use so create new show_vlan function and use textfsm
# def show_vlan(session: object, model: str):
#     ret = session.command_expect('show vlan', parse_colon_fields_structure)
#     return ret['data']

def show_arp(session: object, model: str):
    """
    Execute the 'show vlan' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): Session object to execute the command on. 
            model (str): Model of the device for which to use the TextFSM template. 
            
        Returns: 
            dict: Parsed data from 'show vlan' command in form of a dictionary. 
    """
    filename = os.path.join(
        dirname, f'{model.__name__}/moxa_ng_router_show_arp.textfsm')
    session.command('terminal length 0')
    ret = session.command_expect('show arp',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_mac_address_table(session: object, model: str):
    """
    Execute the 'show mac-address-table' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): Session object to execute the command on. 
            model (str): Model of the device for which to use the TextFSM template. 
            
        Returns: 
            dict: Parsed data from 'show mac-address-table' command in form of a dictionary. 
    """
    filename = os.path.join(
        dirname, f'{model.__name__}/moxa_ng_router_show_mac_address_table.textfsm')
    ret = session.command_expect('show mac-address-table',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_vlan(session: object, model: str):
    """
    Execute the 'show vlan' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): Session object to execute the command on. 
            model (str): Model of the device for which to use the TextFSM template. 
            
        Returns: 
            dict: Parsed data from 'show vlan' command in form of a dictionary. 
    """
    filename = os.path.join(
        dirname, f'{model.__name__}/moxa_ng_router_show_vlan.textfsm')
    # session.command('terminal length 0')
    ret = session.command_expect('show vlan',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_clock(session: object, model: str):
    """
    Execute the 'show clock' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (Session): An established session object to the device.
            model (str): The device model type.

        Returns:
            dict: A dictionary containing the system clock information. 
    """
    ret = session.command_expect('show clock', parse_colon_fields)
    return ret['data']

def show_ip_route(session: object, model: str):
    """
    Execute the 'show ip route' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (Session): An established session object to the device.
            model (str): The device model type.

        Returns:
            list[dict]: A list of dictionaries containing the IP routing table information. 
    """
    from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_ip_route.textfsm')
    if issubclass(model, tn5000):
        ret = session.command_expect('show ip route',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    else:
        session.command('terminal length 0')
        ret = session.command_expect('show ip route',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     exact_str=True,
                                     timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_ip_igmp(session: object, model: str):
    """
    Execute the 'show ip igmp' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (Session): An established session object to the device.
            model (str): The device model type.

        Returns:
            list[dict]: A list of dictionaries containing the IP routing table information. 
    """
    from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_ip_igmp.textfsm')
    if issubclass(model, tn5000):
        ret = session.command_expect('show ip igmp',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    else:
        ret = session.command_expect('show ip igmp',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     exact_str=True,
                                     timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_trusted_access(session: object, model: str):
    """
    Execute the 'show interface trusted-access' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): The session object to connect to the device.
            model (str): The model of the device.

        Returns:
            res (dict): A dictionary containing the trusted access information. 
    """
    from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_trusted_access.textfsm'
    )
    if issubclass(model, tn5000):
        ret = session.command_expect('show interface trusted-access',
                                    prompts=PROMPTS["ALL"]["MAIN"],
                                    exact_str=True,
                                    timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    else:
        # session.command('terminal length 0')
        ret = session.command_expect('show interface trusted-access',
                                    prompts=PROMPTS["ALL"]["MAIN"],
                                    exact_str=True,
                                    timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_l37_policy(session: object, model: str):
    """
    Execute the 'show l3l7-policy' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): The session object to connect to the device.
            model (str): The model of the device.

        Returns:
            res (dict): A dictionary containing the L3/L7 policy information. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_l3l7-policy.textfsm')
    session.command('terminal length 0')
    ret = session.command_expect('show l3l7-policy',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_vrrp(session: object, model: str):
    """
    Execute the 'show vrrp detail' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): The session object to connect to the device.
            model (str): The model of the device.

        Returns:
            res (dict): A dictionary containing the vrrp information. 
    """
    from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_vrrp.textfsm')
    if issubclass(model, tn5000):
        ret = session.command_expect('show ip vrrp',
                                    prompts=PROMPTS["ALL"]["MAIN"],
                                    exact_str=True,
                                    timeout=5)
    else:
        ret = session.command_expect('show vrrp detail',
                                    prompts=PROMPTS["ALL"]["MAIN"],
                                    exact_str=True,
                                    timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_logging_event_log_dos(session: object, model: str):
    """
    Execute the 'show logging event-log dos' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): The session object to connect to the device.
            model (str): The model of the device.

        Returns:
            res (dict): A dictionary containing DOS event log information.
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_logging_event_log_dos.textfsm'
    )
    session.command('terminal length 0')
    ret = session.command_expect('show logging event-log dos',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_logging_event_log_l3l7(session: object, model: str):
    """
    Execute the 'show logging event-log l3l7-policy' command on a Moxa NG router and parse the output using TextFSM.

    Args: 
        session (object): The session object used to execute the command. 
        model (str): The model of the router. 

    Returns: 
        res (dict): A dictionary containing the parsed output of the command.
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_logging_event_log_l3l7-policy.textfsm'
    )
    session.command('terminal length 0')
    ret = session.command_expect('show logging event-log l3l7-policy',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    session.command_expect('', prompts=PROMPTS["ALL"]["MAIN"])
    return res


def show_logging_event_log_firewall(session: object, model: str):
    """
    Execute the 'show logging event-log firewall' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): A Session object to execute commands on.
            model (class): The model class to use for the device.
            
        Returns:
            list: A list of dictionaries containing the parsed output from the command. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_logging_event_log_firewall.textfsm'
    )
    ret = session.command_expect('show logging event-log firewall',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    session.command_expect('', prompts=PROMPTS["ALL"]["MAIN"])
    return res


def show_logging_event_log_system(session: object, model: str):
    """
    Execute the 'show users' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): Paramiko SSH session object. 
            model (str): Model of the Moxa NG router. 
            
        Returns: 
            res (dict): A dictionary containing the user accounts information. 
    """
    from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_logging_event_log_system.textfsm'
    )
    time.sleep(5)
    if issubclass(model, tn5000):
        logs = ""
        ret = session.command_expect('show logging event-log system',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     timeout=5)
        logs += ret['data']
        while ret['matched'] == False:
            ret = session.command_expect('',
                                         prompts=PROMPTS["ALL"]["MAIN"],
                                         timeout=1)
            logs += ret['data']
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(logs.replace('\r\n--More--', '\n'))
    else:
        session.command('terminal length 0')
        ret = session.command_expect('show logging event-log system',
                                     prompts=PROMPTS["ALL"]["MAIN"],
                                     exact_str=True,
                                     timeout=5)
        with open(filename) as template:
            fsm = textfsm.TextFSM(template)
            res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_logging_event_log_malformed(session: object, model: str):
    """
    Execute the 'show logging event-log malformed' command on a Moxa NG router and parse the output using TextFSM.

        Args:
            session (object): A Session object to execute commands on.
            model (class): The model class to use for the device.
            
        Returns:
            list: A list of dictionaries containing the parsed output from the command. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_logging_event_log_malformed.textfsm'
    )
    ret = session.command_expect('show logging event-log malformed',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res
    

def show_user_accounts(session: object, model: str):
    """
    Execute the 'show users' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): Paramiko SSH session object. 
            model (str): Model of the Moxa NG router. 
            
        Returns: 
            res (dict): A dictionary containing the user accounts information. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_user_accounts.textfsm')
    ret = session.command_expect('show users',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


def show_object(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_object.textfsm')
    ret = session.command_expect('show object',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    res = list(map(lambda x: dict(x, **{'detail': x['detail'].rstrip()}), res))
    return res


def show_session_ctrl(session: object, model: str):
    filename = os.path.join(dirname,
        f'{model.__name__}/moxa_ng_router_show_session_ctrl.textfsm')
    ret = session.command_expect('show session-control',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
        res = list(
            map(lambda x: dict(x, **{'session_name': x['session_name'].rstrip(), 'index': x['index'].rstrip()}), res))
    return res

def show_lldp_table(session: object, model:str):
    filename = os.path.join(dirname,
        f'{model.__name__}/moxa_ng_router_show_lldp_table.textfsm')
    ret = session.command_expect('show lldp entry',
                                 prompts=[r'# '],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
        res = list(map(lambda item: {k: v.rstrip() for k, v in item.items()}, res))
    return res

def show_rstp(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_rstp.textfsm')
    ret = session.command_expect('show redundancy spanning-tree',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_rstp_port(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_rstp_port.textfsm')
    ret = session.command_expect('show redundancy spanning-tree',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    # spilt ret to parse port information
    split_index = ret['data'].find("Int#")
    ret['data'] = ret['data'][split_index:]

    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_ip_ospf_interface(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_ip_ospf_interface.textfsm')
    ret = session.command_expect('show ip ospf interface',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_ip_ospf_neighbor(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_ip_ospf_neighbor.textfsm')
    ret = session.command_expect('show ip ospf neighbor',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_ip_ospf_database(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_ip_ospf_database.textfsm')
    ret = session.command_expect('show ip ospf database',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_l2tp_setting(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_l2tp.textfsm')
    ret = session.command_expect('show l2tp',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_snmp(session: object, model: str):
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_snmp.textfsm')
    ret = session.command_expect('show snmp',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=5)
    
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res

def show_fast_bootup(session: object, model: str):
    """
    Execute the 'show version' command on a Moxa NG router and parse the output using TextFSM.

        Args: 
            session (object): The session object to use for connecting to the device. 
            model (str): The model of the device. 
            
        Returns: 
            res (list): A list of dictionaries containing the version information. 
    """
    filename = os.path.join(
        dirname,
        f'{model.__name__}/moxa_ng_router_show_fast_bootup.textfsm')
    ret = session.command_expect('show fast-bootup',
                                 prompts=PROMPTS["ALL"]["MAIN"],
                                 exact_str=True,
                                 timeout=1)
    with open(filename) as template:
        fsm = textfsm.TextFSM(template)
        res = fsm.ParseTextToDicts(ret['data'])
    return res


import os
import time

from host import *
from behave import *


@when('"{host_send}" send {count} ICMP response to "{host_recv}" through "{device}"')
def step_impl(context, host_send, count, host_recv, device):
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host_send]['session']['credential']['password'],
                                                            "../lib/replay_pkts/icmp_reply.pcapng",
                                                            context.host_info[host_send]['session']['credential']['username'],
                                                            context.host_info[host_send]['session']['host']))
    dut_mac = context.dut[device].main().show_system()[0].pop("mac_address")
    context.hosts[host_send].tcp_tool.rewrite(infile="icmp_reply.pcapng", outfile="icmp_reply.pcap",
                                                sip=context.host_info[host_send]["testbed"]["cur_host"],
                                                dip=context.host_info[host_recv]["testbed"]["cur_host"], 
                                                smac=context.host_info[host_send]["testbed"]["mac_address"],
                                                dmac=dut_mac)
    context.hosts[host_send].tcp_tool.send(sendfile="icmp_reply.pcap", number=count)
    time.sleep(5)


@then('"{host_recv}" shall receive {count} ICMP reponse from "{host_send}"')
def step_impl(context, host_recv, count, host_send):
    context.hosts[host_recv].tshark.stop()

    ret = context.hosts[host_recv].tshark.retrieve()

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    ret_lines = ret.split('\n')

    destip_count = {}
    # parse and get dest IP counts
    for a in ret_lines:
        # skip if not ICMP
        if a.find("ICMP") == -1:
            continue

        arr = a.lstrip(' ').split(' ')

        # skip unexpected format
        if arr[3] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        if arr[4] in destip_count:
            destip_count[arr[4]] += 1
        else:
            destip_count[arr[4]] = 1

    # compare dest ip count
    host_recv_ip = context.host_info[host_recv]['testbed']['cur_host']

    if host_recv_ip not in destip_count:
        recv_packet_count = 0
    else:
        recv_packet_count = destip_count[host_recv_ip]

    assert int(recv_packet_count) == int(
        count
    ), f'{host_recv} expect to receive {count} packets but receive {recv_packet_count}'


@when('malformed packets is "{action}" on "{device}"')
def step_impl(context, action, device):
    context.dut[device].go_config().set_malformed_packet_status(status=action)


@when('malformed packets logging is "{action}" on "{device}"')
def step_impl(context, action, device):
    context.dut[device].go_config().set_malformed_packet_log(status=action, dst={"flash"})


@then('malformed event log shall not be recorded on "{device}"')
def step_impl(context, device):
    r = context.dut[device].main().show_logging_event_log_malformed()
    # print(r)
    assert len(r) == 0, f'expect no record but {len(r)} log was/were recorded'


@then('{count} malformed event log shall be recorded on "{device}"')
def step_impl(context, count, device):
    r = context.dut[device].main().show_logging_event_log_malformed()
    assert len(r) == int(count), f'expect {count} record but {len(r)} log was/were recorded'


from typing import Any

from mdc.router.cli.md_tn5916.tn5916_v3 import tn5916_v3_0
from mdc.router.cli.md_tn5916.tn5916_v3_0_27 import tn5916_v3_0_27
from mdc.router.cli.md_edr810.edr810_v5 import edr810_v5_0
from mdc.router.cli.md_edr8010.edr8010_v3 import edr8010_v3_0
from mdc.router.cli.md_edrg9010.edrg9010_v2 import edrg9010_v2_0
from mdc.router.cli.md_edrg9010.edrg9010_v3 import edrg9010_v3_0
# from mdc.router.cli.md_oncellg4302.oncellg4302_v3 import oncellg4302_v3_0

class Router:
    
    def __init__(self, session, model):
        print(f'You are using Router {model} lib')
        self._s = session
        self._model = eval(model)
        self._model_name = model

    # ==== Router API portion ==== #
    def close(self):
        self._s.close()

    def main(self):
        return self._model.Main(self._s, self._model)

    def go_config(self):
        return self._model.Config(self._s)

    def go_config_if_lan(self, *args, **kwargs):
        return self._model.ConfigIfLan(self._s, *args, **kwargs).getin()

    def go_config_if_wan(self, *args, **kwargs):
        return self._model.ConfigIfWan(self._s, *args, **kwargs).getin()

    def go_config_if_vlan(self, *args, **kwargs):
        return self._model.ConfigIfVlan(self._s, *args, **kwargs).getin()

    def go_config_if_ethernet(self, *args, **kwargs):
        return self._model.ConfigIfEthernet(self._s, *args, **kwargs).getin()

    def go_config_bridge(self, *args, **kwargs):
        return self._model.ConfigBrg(self._s, *args, **kwargs).getin()

    def go_config_l2filter(self, *args, **kwargs):
        return self._model.ConfigL2filter(self._s, *args, **kwargs).getin()

    def go_config_ipsec(self, *args, **kwargs):
        return self._model.ConfigIpsec(self._s, *args, **kwargs).getin()

    def go_config_object_addr(self, *args, **kwargs):
        return self._model.ConfigObjectAddr(self._s, *args, **kwargs).getin()

    def go_config_object_serv(self, *args, **kwargs):
        return self._model.ConfigObjectServ(self._s, *args, **kwargs).getin()

    def go_config_object_net_serv(self, *args, **kwargs):
        return self._model.ConfigObjectNetServ(self._s, *args, **kwargs).getin()

    def go_config_object_industrial_app_serv(self, *args, **kwargs):
        return self._model.ConfigObjectIndustrialAppServ(self._s, *args, **kwargs).getin()

    def go_config_l37policy(self, *args, **kwargs):
        return self._model.ConfigL37Policy(self._s, *args, **kwargs).getin()
    
    def go_config_session_ctrl(self, *args, **kwargs):
        return self._model.ConfigSessionControl(self._s, *args, **kwargs).getin()

    def go_config_nat(self, *args, **kwargs):
        return self._model.ConfigNat(self._s, *args, **kwargs).getin()

    def go_config_dhcp(self, *args, **kwargs):
        return self._model.ConfigDhcp(self._s, *args, **kwargs).getin()

    def go_config_firewall(self, *args, **kwargs):
        return self._model.ConfigFirewall(self._s, *args, **kwargs).getin()

    def go_config_ospf(self, *args, **kwargs):
        return self._model.ConfigOspf(self._s, *args, **kwargs).getin()

    def go_config_vrrp(self, *args, **kwargs):
        return self._model.ConfigVrrp(self._s, *args, **kwargs).getin()

    def go_config_rstp(self, *args, **kwargs):
        return self._model.ConfigRstp(self._s, *args, **kwargs).getin()
    
    def go_config_rip(self, *args, **kwargs):
        return self._model.ConfigRip(self._s, *args, **kwargs).getin()


from utils.input_chk import chk_valid_ip
from mdc.router.cli.rp_base.lg_router.edr800 import cli
from mdc.router.cli.rp_base.lg_router.edr800.cli import PROMPT_CONFIG

class Config(cli.Cli):
    """
    Class for configuring a device through the command line interface.

        Args:
            session (Session): A Session object used to communicate with the device.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
    """

    def __init__(self, session):
        super().__init__(session)
        # back to main
        self._back_to_main()
        # jumpt to config terminal
        self.getin_cfg()

    def get_session(self):
        return self._s

    def _set_default_prompts_cfg(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin_cfg(self):
        self._s.command_expect('configure terminal')

    # function override is allowed
    def getin_cfg(self):
        self._set_default_prompts_cfg()
        self._getin_cfg()
        return self

    def set_fast_bootup(self, global_toggle=None):
        if global_toggle:
            if global_toggle == "enable":
                self.command(f"fast-bootup")
            elif global_toggle == "disable":
                self.command(f"no fast-bootup")
            else:
                raise ValueError(
                    f'input invalid global_toggle: {global_toggle}')



import time
from datetime import datetime, timedelta

from behave import *

@given('set summer time to start at "{start_time}" on "{device}"')
def step_impl(context, start_time, device):
	context.dut[device].go_config().set_clock_summer_time(start_date=start_time)


@given('set summer time to end at "{end_time}" on "{device}"')
def step_impl(context, end_time, device):
	context.dut[device].go_config().set_clock_summer_time(end_date=end_time)


@given('set summer time offset as "{offset}" hour on "{device}"')
def step_impl(context, offset, device):
	context.dut[device].go_config().set_clock_summer_time(offset=offset)


@when('set the current time on "{device}" {min} minute behind "{real_start_time}"')
def step_impl(context, device, min, real_start_time):

    # Parse the original date string
    original_time = datetime.strptime(real_start_time, "%a %B %d %H:%M:%S %Y")
    # Format the date as "02:00:00 3 12 2023"
    # formatted_date = original_time.strftime("%H:%M:%S %m %d %Y")
    
    new_time = original_time - timedelta(minutes=1)
    cfg = context.dut[device].go_config()
    cfg.TimeSync(cfg.get_session()).set_time_manually(new_time.year, new_time.month, new_time.day, \
        new_time.hour, new_time.minute, new_time.second)


@when('wait {min} minute until the current time on "{device}" reaches "{start_time}"')
def step_impl(context, min, device, start_time):
    time.sleep(int(min)*60)


@then('the current time on "{device}" should be {offset} hour ahead of "{real_start_time}"')
def step_impl(context, device, offset, real_start_time):
    ret = context.dut[device].main().show_clock()
    clock_time = ret['Current Time']

    clock_datetime = datetime.strptime(clock_time, "%a %b %d %H:%M:%S %Y")
    real_start_datetime = datetime.strptime(real_start_time, "%a %B %d %H:%M:%S %Y")
    time_difference = clock_datetime - real_start_datetime

    # Check if the time difference is within 1 hour +1/-0 minute 
    is_within = timedelta(hours=1) <= time_difference <= timedelta(hours=1, minutes=1)

    assert is_within, f'the clock_datetime is {clock_datetime} while real_start_datetime is {real_start_datetime}'


@then('the current time on "{device}" should be {offset} hour behind "{real_end_time}"')
def step_impl(context, device, offset, real_end_time):
    ret = context.dut[device].main().show_clock()
    clock_time = ret['Current Time']

    clock_datetime = datetime.strptime(clock_time, "%a %b %d %H:%M:%S %Y")
    real_end_datetime = datetime.strptime(real_end_time, "%a %B %d %H:%M:%S %Y")
    time_difference = real_end_datetime - clock_datetime

    # Check if the time difference is within 1 hour +0/-1 minute
    is_within = timedelta(hours=1, minutes=-1) <= time_difference <= timedelta(hours=1)

    assert is_within, f'the clock_datetime is {clock_datetime} while real_end_datetime is {real_end_datetime}'


from session import SshSession

class TrexConsoleService:
    """
    Class to control the TRex console.

        Args:
            ssh_session (SshSession): An SSH session object.
            sudo_paswd (str): The sudo password for the remote server.

        Attributes:
            _ssh (SshSession): SSH session object.
            _trex_path (str): Path to TRex directory.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._trex_path = '/home/rmd/trex/v2.99'

    def check_env(self):
        """
        Checks if the TRex server is running and if the TRex path is valid.

            Raises:
                ValueError: If TRex server is not running or TRex path is not valid.
        """
        output = self._ssh.command('ps -aux | grep trex')
        if 'scapy_server' not in output:
            raise ValueError('TRex Server not start')

        output = self._ssh.command(f'ls {self._trex_path}')
        if 'No such file or directory' in output:
            raise ValueError(f'TRex not found at {self._trex_path}')

    def start(self):
        """
        Start trex console.

            Raises:
                ValueError: If trex console fails to start or cannot find prompt.
        """
        self._ssh.command(f'cd {self._trex_path}')
        ret = self._ssh.command('./trex-console',
                                exact_prompts=['trex>'],
                                timeout=60)
        if ret is None:
            raise ValueError('Start trex console fail. Cannot find prompt')
        if ret.count('[SUCCESS]') != 3:
            raise ValueError(f'Start trex console error. Output:\n{ret}')

    def send_packet(self, packet_path: str, rate: int, unit: str, ports: list):
        """
        Send a packet to the specified ports.

            Args:
                packet_path (str): Path of the packet.
                rate (int): Rate of the packet.
                unit (str): Unit of the rate, eg: 'pps'.
                ports (list[int]): List of ports to send the packet. 
                
            Raises: 
                ValueError: If sending start command fails. 
        """
        port = ' '.join([str(p) for p in ports])
        # eg: start -f stl/icmpv6.py -m 10pps --port 0
        cmd = f'start -f {packet_path} -m {rate}{unit} --port {port}'
        ret = self._ssh.command(cmd, exact_prompts=['trex>'])
        if ret.count('[SUCCESS]') != 3:
            raise ValueError(f'Send start command error. Output:\n{ret}')

    def stop(self):
        """
        Stop the TRex traffic generator.
        """
        self._ssh.command('stop', exact_prompts=['trex>'])
        self._ssh.command('clear', exact_prompts=['trex>'])
        self._ssh.command('exit')


class TrexServerService:
    """
    Class for controlling a TRex server over SSH.

        Args:
            ssh_session (SshSession): An SSH session object.
            sudo_paswd (str): The sudo password for the remote server.

        Attributes:
            _ssh (SshSession): An SSH session object.
            _trex_path (str): The path to the TRex installation on the remote server.
            _screen_pid (int, optional): The process ID of the TRex screen session, if one is running. Defaults to None.
            _sudo_pwd (str): The sudo password for the remote server.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._trex_path = '/home/rmd/trex/v2.99'
        self._screen_pid = None
        self._sudo_pwd = sudo_paswd

    def check_env(self):
        """
        Check if TRex is installed in the given path.

            Raises:
                ValueError: If TRex is not found at the given path.
        """
        output = self._ssh.command(f'ls {self._trex_path}')
        if 'No such file or directory' in output:
            raise ValueError(f'TRex not found at {self._trex_path}')

    def start(self):
        """
        Start TRex on the remote server.

            Raises:
                ValueError: If wait server starts timeout.
            Usage:
                1. change dir to trex folder because there are related config files
                2. screen
                3. sudo ./t-rex-64 -i
                4. space
                5. ctrl + A D
        """
        self._ssh.command(f'cd {self._trex_path}')
        # NOTE: how to clean existed screen -> killall screen
        self._ssh.command(
            'screen',
            exact_prompts=['[Press Space for next page; Return to end.'])
        self._ssh.command(' ')

        ret = self._ssh.command(
            f'echo {self._sudo_pwd} | sudo -S ./t-rex-64 -i',
            exact_prompts=['test duration'],
            timeout=60)

        if ret is None:
            raise ValueError('Wait server starts timeout')

        self._ssh.sendcontrol('A')
        self._ssh.sendcontrol('D')

        output = self._ssh.command('')  # expect prompt to get output
        idx = output.find('[detached from ')
        msg = output[idx:]  # eg: should be [detached from 27397.pts-4.trex]
        name = msg.split()[-1]
        self._screen_pid = name.split('.')[0]

    def stop(self):
        """
        Stop the server.

            Usage:
                1. Change to working dir
                2. screen -R 27397
                3. send ctrl + C 
                4. exit.
        """
        self._ssh.command(f'screen -R {self._screen_pid}',
                          exact_prompts=['test duration'])
        # self._ssh.command('', exact_prompts=['test duration'])
        self._ssh.sendcontrol('C')
        self._ssh.command('')  # wait server stopped
        self._ssh.command('exit')  # leave screen


class TrexService:
    """
    Class for controlling TRex server.

        Args:
            ssh_session (SshSession): SSH session object.
            sudo_paswd (str): Password for sudo user.

        Attributes:
            _ssh (SshSession): SSH session object.
            _trex_path (str): Path to TRex server.
            _screen_pid (int): PID of the screen process running TRex server.
            _sudo_pwd (str): Password for sudo user.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._trex_path = '/home/rmd/trex/v2.99'
        self._screen_pid = None
        self._sudo_pwd = sudo_paswd

    def check_env(self):
        """
        Check if TRex is installed in the given path.

            Raises:
                ValueError: If TRex is not found at the given path.
        """
        output = self._ssh.command(f'ls {self._trex_path}')
        if 'No such file or directory' in output:
            raise ValueError(f'TRex not found at {self._trex_path}')

    def _start_server(self):
        """
        Start the TRex server on the remote machine.

            Raises:
                ValueError: If waiting for server start times out or an error occurs while starting the server. 
            
            Usage:
                1. change dir to trex folder because there are related config files
                2. screen
                3. sudo ./t-rex-64 -i
                4. space
                5. ctrl + A D
        """
        self._ssh.command(f'cd {self._trex_path}')
        # NOTE: how to clean existed screen -> killall screen
        self._ssh.command(
            'screen',
            exact_prompts=['[Press Space for next page; Return to end.'])
        self._ssh.command(' ')

        ret = self._ssh.command(
            f'echo {self._sudo_pwd} | sudo -S ./t-rex-64 -i',
            exact_prompts=['test duration'],
            timeout=60)

        if ret is None:
            raise ValueError('Wait server starts timeout')

        self._ssh.sendcontrol('A')
        self._ssh.sendcontrol('D')

        output = self._ssh.command('')  # expect prompt to get output
        idx = output.find('[detached from ')
        msg = output[idx:]  # eg: should be [detached from 27397.pts-4.trex]
        name = msg.split()[-1]
        self._screen_pid = name.split('.')[0]

    def _stop_server(self):
        """
        Stops the TRex server by sending the appropriate commands to the SSH connection.

            Usage:
                1. change to working dir
                2. screen -R 27397
                3. ctrl + C
                4. exit
        """
        self._ssh.command(f'screen -R {self._screen_pid}',
                          exact_prompts=['test duration'])
        # self._ssh.command('', exact_prompts=['test duration'])
        self._ssh.sendcontrol('C')
        self._ssh.command('')  # wait server stopped
        self._ssh.command('exit')  # leave screen

    def _start_console(self):
        """
        Start the TRex console.

            Raises:
                ValueError: If start trex console fail or output is not success.
        """
        self._ssh.command(f'cd {self._trex_path}')
        ret = self._ssh.command('./trex-console',
                                exact_prompts=['trex>'],
                                timeout=60)
        if ret is None:
            raise ValueError('Start trex console fail. Cannot find prompt')
        if ret.count('[SUCCESS]') != 3:
            raise ValueError(f'Start trex console error. Output:\n{ret}')

    def _send_packet(self, packet_path: str, rate: str, unit: str, ports: list):
        """
        Send a packet to the specified ports.

            Args:
                packet_path (str): Path of the packet.
                rate (str): Rate of the packet.
                unit (str): Unit of the rate, such as 'pps'.
                ports (list): List of ports. 

            Raises:
                ValueError: If sending start command fails. 
        """
        port = ' '.join([str(p) for p in ports])
        # eg: start -f stl/icmpv6.py -m 10pps --port 0
        cmd = f'start -f {packet_path} -m {rate}{unit} --port {port}'
        ret = self._ssh.command(cmd, exact_prompts=['trex>'])
        if ret.count('[SUCCESS]') != 3:
            raise ValueError(f'Send start command error. Output:\n{ret}')

    def _stop_console(self):
        """
        Stop the TRex console.
        """
        self._ssh.command('stop', exact_prompts=['trex>'])
        self._ssh.command('exit')

    def send(self, packet_path: str, rate: str, unit: str, port: str):
        """
        Send a packet to the server.

            Args:
                packet_path (str): Path of the packet to send.
                rate (str): Rate of the packet to send.
                unit (str): Unit of rate, e.g., 'bps'.
                port (str): Port number to send the packet.
             
        """
        self._start_server()
        self._start_console()
        self._send_packet(packet_path, rate, unit, port)

    def stop(self):
        """
        Stop the console and server.
        """
        self._stop_console()
        self._stop_server()



from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigIfWan(config.Config):
    """
    Class for configuring WAN interface.

        Args:
            session (Session): Session object to communicate with the device.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (tuple): Tuple of positional arguments.
            _kwargs (dict): Dictionary of keyword arguments. 
    """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        """
        Get into the interface wan.
        """
        self._s.command_expect(f'interface wan')

    def getin(self):
        """
        Set the default prompts and get into the interface wan.

            Returns: 
                self: The instance of the object. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def set_wan_interface(self,
                            ip: str,
                            mask: str,
                            bind_vlan: str = None,
                            mac_address: str = None,
                            gateway: str = None):
        """
        Set the WAN interface with given IP address and subnet mask.

            Args:
                ip (str): IP address of the WAN interface.
                mask (str): Subnet mask of the WAN interface.
                bind_vlan (str, optional): VLAN to bind to the WAN interface. Defaults to None.
                mac_address (str, optional): MAC address of the WAN interface. Defaults to None.

            Returns:
                self: The object instance itself.
        """
        if gateway:
            self._s.command_expect(
                f'ip address static {ip} {mask} {gateway}')
        else:
            self._s.command_expect(f'ip address static {ip} {mask}')
        if bind_vlan:
            self._s.command_expect(f'bind vlan {bind_vlan}')
        if mac_address:
            self._s.command_expect(f'mac-address {mac_address}')
        return self

    def disable_wan_interface(self):
        """
        Disable the WAN interface.

            Returns: 
                self: The object instance itself. 
        """
        self._s.command_expect(f'shutdown')
        return self

    def ip_address_static(self, ip: str, mask: str, gateway: str = ''):
        """
        Set a static IP address for the WAN interface with given subnet mask and gateway (optional).

            Args: 
                ip (str): IP address of the WAN interface. 
                mask (str): Subnet mask of the WAN interface. 
                gateway (str, optional): Gateway for the WAN interface. Defaults to ''.

            Returns: 
                self: The object instance itself.  
        """
        self._s.command_expect(f'ip address static {ip} {mask} {gateway}')
        return self

    def ip_address_dhcp(self, bind_vlan):
        """
        Set a WAN interface with dhcp mode.

            Returns: 
                self: The object instance itself.  
        """
        self._s.command_expect(f'bind vlan {bind_vlan}')
        self._s.command_expect(f'ip address dhcp')
        return self

    def set_pptp(self, ip, username, pwd):
        self._s.command_expect(f'ip pptp {ip} {username} {pwd}')
        return self

    def set_dns(self,
                dns_server_1: str = '8.8.8.8',
                dns_server_2: str = '10.123.200.11',
                dns_server_3: str = '10.123.200.12'):

        self._s.command_expect(
            f'ip name-server {dns_server_1} {dns_server_2} {dns_server_3}')
        return self



import time
import random
from behave import *
from steps import common, interface, login

@when(u'send {pkt_count} rip {rip_type} packet with the content of the table from "{host}" to "{device}"')
def step_impl(context, pkt_count, rip_type, host, device):
    ripentries = []
    for row in context.table:
        row_dict = {}
        for heading, value in row.items():
            if heading == "nexthop":
                row_dict[heading] = context.host_info[host]["testbed"]["cur_host"]
            else:
                row_dict[heading] = value
        ripentries.append(row_dict)
    context.ripentries = ripentries
    
    print(ripentries)
    script = context.hosts[host].scapy.generate_rip_scapy_script(rip_type = rip_type,
                                                                 sip=context.host_info[host]["testbed"]["cur_host"],
                                                                 dip=context.host_info[host]["testbed"]["cur_gw"],
                                                                 smac=context.host_info[host]["testbed"]["mac_address"],
                                                                 dmac="01:00:5E:00:00:09",
                                                                 iface=context.host_info[host]["nic"],
                                                                 ripentries=ripentries 
                                                                )
    from datetime import datetime
    import os

    # write the scapy script to file
    scapy_file_name = datetime.now().isoformat().replace(":", "_")[:19]
    with open(f'../lib/atb/service/scapy_template/{scapy_file_name}', "w") as script_file:
        script_content = script_file.write(script)
    
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host]['session']['credential']['password'],
                                                            f"../lib/atb/service/scapy_template/{scapy_file_name}",
                                                            context.host_info[host]['session']['credential']['username'],
                                                            context.host_info[host]['session']['host']))
    # Clean up
    for _ in range(int(pkt_count)):
        context.hosts[host].scapy.do_script(scapy_file_name)
    os.system(f'rm ../lib/atb/service/scapy_template/{scapy_file_name}')

@when(u'set rip with below table on "{device}"')
def step_impl(context, device):
    network = context.table[0]["network"].strip().split(",")
    version = context.table[0]["version"]
    redistribute = context.table[0]["redistribute"]
    print("network:", network)
    print("version:", version)
    print("redistribute:", redistribute)
    context.dut[device].go_config_rip().set_rip(network=network, 
                                                version=version, 
                                                redistribute=redistribute
                                                )

@then(u'routing table should be updated with response on "{device}"')
def step_impl(context, device):
    flag = False
    ret = context.dut[device].main().show_ip_route()
    for rip_entry in context.ripentries:
        expect_addr = rip_entry['addr']
        expect_mask = rip_entry['mask']
        expect_nexthop = rip_entry['nexthop']
        expect_metric = str(int(rip_entry['metric']) + 1)

        binary_mask = ''.join(format(int(x), '08b') for x in expect_mask.split('.'))
        expect_mask = str(len(binary_mask.rstrip('0')))
        if any((actual_entry['destination'] == f"{expect_addr}/{expect_mask}" \
               and actual_entry['next_hop'] == expect_nexthop \
               and actual_entry['metric'] == expect_metric) for actual_entry in ret):
            flag = True
    
    assert flag, f"routing rule is not updated ->\n expect: {context.ripentries}\n actual: {ret}"


import re
import sys
import time

from host import *
from behave import *
from steps import common, interface, login
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000


@when(u'send traffic with dos_type {dos_type} from "{host1}" to "{host2}"')
def step_impl(context, dos_type, host1, host2):
    command_dict = {
        "Null Scan": ['null'],
        "Xmas Scan": ['fin', 'urg', 'push', 'syn', 'rst', 'ack'],
        "Nmap-ID Scan": ['fin', 'urg', 'push', 'syn'],
        "SYN-FIN Scan": ['fin', 'syn'],
        "FIN Scan": ['fin'],
        "SYN-RST Scan": ['syn', 'rst'],
        "Nmap-Xmas Scan": ['fin', 'urg', 'push'],
        "New-TCP-without-SYN Scan": ['ack'],
        "New-without-SYN Scan": ['ack'],
    }
    context.hosts[host2].tshark.start(interface=context.host_info[host2]['nic'])
    time.sleep(5)
    dos_type = dos_type.replace('/', '-')
    context.hosts[host1].hping3.send( \
        tcpudp_flags=command_dict[dos_type], \
        host = context.host_info[host2]['testbed']['cur_host'], \
        count = 4)
    time.sleep(5)


@then(
    u'check if "{host}" {can_or_cannot} receive traffic with dos_type {dos_type}'
)
def step_impl(context, can_or_cannot, dos_type, host):
    # ['ICMP' , 'Flags [S]', 'UDP']
    flag = { \
        'can': True, \
        'cannot': False}[can_or_cannot]
    command_dict = {
        "Null Scan":
            '<None>',
        "Xmas Scan": (', ').join(['fin', 'syn', 'rst', 'psh', 'ack',
                                  'urg']).upper(),
        "Nmap-ID Scan": (', ').join(['fin', 'syn', 'psh', 'urg']).upper(),
        "SYN-FIN Scan": (', ').join(['fin', 'syn']).upper(),
        "FIN Scan": ('').join(['fin']).upper(),
        "SYN-RST Scan": (', ').join(['syn', 'rst']).upper(),
        "Nmap-Xmas Scan": (', ').join(['fin', 'psh', 'urg']).upper(),
        "New-TCP-without-SYN Scan": ('').join(['ack']).upper(),
        "New-without-SYN Scan": ('').join(['ack']).upper(),
    }
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve()
    print(ret)
    dos_type = dos_type.replace('/', '-')
    count = 0
    if ret != None:
        count = ret.count(f'{command_dict[f"{dos_type}"]}')
    if flag:
        assert count >= 1, f'expect receive 4 number of {command_dict[f"{dos_type}"]} but {count}'
    else:
        assert count == 0, f'expect receive 0 number of {command_dict[f"{dos_type}"]} but {count}'


@then(
    u'check traffic with {dos_type} from {incoming_interface} ("{host1}") to {outgoing_interface} ("{host2}") can be blocked and logged by "{device}"'
)
def step_impl(context, dos_type, incoming_interface, outgoing_interface, device,
              host1, host2):
    if issubclass(context.dut[device]._model, tn5000):
        ret = context.dut[device].main().show_logging_event_log_firewall()
    else:
        ret = context.dut[device].main().show_logging_event_log_dos()
    count = 0
    if incoming_interface.upper() == 'BRG' and outgoing_interface.upper(
    ) == 'BRG':
        incoming_interface, outgoing_interface = 'BRG_LAN', 'BRG_LAN'
    log_expect = ['TCP', incoming_interface.upper(), context.host_info[host1]['testbed']['cur_host'], outgoing_interface.upper(), \
           context.host_info[host2]['testbed']['cur_host'], 'DROP']
    log_actual = []
    for r in ret:
        log_actual = [
            r['protocol'], r['in'], r['src_ip'], r['out'], r['dest_ip'],
            r['action']
        ]
        if log_expect == log_actual:
            count += 1
    if dos_type.upper() == 'ICMP':
        assert count == 1, f'logged event-log dismatch: expect 1 but {count}, {log_expect} <-> {log_actual}'
    else:
        assert count == 4, f'logged event-log dismatch: expect 4 but {count}, {log_expect} <-> {log_actual}'



import time

from host import *
from behave import *
from steps import common, interface, login


@given('enable telnet service on "{device}"')
@when('enable telnet service on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_global_telnet(port=23)


@when(
    'add and enable rule "{ip}" with "{netmask}" on "{device}" trusted-access list'
)
def step_impl(context, ip, netmask, device):
    context.dut[device].go_config().set_trusted_access(ip=ip,
                                                       mask=netmask,
                                                       action="enable")
    time.sleep(5)

@when(
    'add and enable rule for "{host}" on "{device}" trusted-access list'
)
def step_impl(context, host, device):
    context.dut[device].go_config().set_trusted_access(ip=context.host_info[host]['testbed']['host'],
                                                       mask='255.255.255.0',
                                                       action="enable")
    time.sleep(5)


@when('{action} rule "{ip}" with "{netmask}" on "{device}"')
def step_impl(context, ip, netmask, device, action):
    context.dut[device].go_config().set_trusted_access(ip=ip,
                                                       mask=netmask,
                                                       action=action)
    time.sleep(3)


@then(
    '"{host}" {action} to access "{device}" with "{protocol}" from {interface}')
def step_impl(context, host, action, device, protocol, interface):
    flag = {'success': True, 'fail': False}[action]

    for row in context.table:
        cmd = {
            "HTTP": {
                "LAN":
                    f'curl -m 10 -I {context.dut_info[device]["testbed"]["lan_ip"]} --connect-timeout 10',
                "WAN":
                    f'curl -m 10 -I {context.dut_info[device]["testbed"]["wan_ip"]} --connect-timeout 10'
            },
            "HTTPS": {
                "LAN":
                    f'curl -m 10 -I https://{context.dut_info[device]["testbed"]["lan_ip"]} --connect-timeout 10',
                "WAN":
                    f'curl -m 10 -I https://{context.dut_info[device]["testbed"]["wan_ip"]} --connect-timeout 10'
            },
            "Telnet": {
                "LAN":
                    f'telnet {context.dut_info[device]["testbed"]["lan_ip"]}',
                "WAN":
                    f'telnet {context.dut_info[device]["testbed"]["wan_ip"]}'
            },
            "SSH": {
                "LAN":
                    f'ssh admin@{context.dut_info[device]["testbed"]["lan_ip"]}',
                "WAN":
                    f'ssh admin@{context.dut_info[device]["testbed"]["wan_ip"]}'
            },
        }[row['protocol']][interface]

        print(cmd)

        if flag:
            if row['protocol'] == 'HTTP':
                ret = context.hosts[host].shellcmd._ssh.command(cmd,
                                                           timeout=5).lower()
                assert any(map(lambda s: s in ret.lower(), \
                               ["http/1.1 200 ok", "http/1.1 308 permanent redirect", "http/1.0 302 redirect"])), \
                                f'[HTTP shall succeed but fail]: ret = {ret}'
            if row['protocol'] == 'HTTPS':
                ret = context.hosts[host].shellcmd._ssh.command(cmd,
                                                           timeout=5).lower()
                assert "ssl certificate problem" in ret, \
                f'[HTTPS shall succeed but fail]: {ret}'
            if row['protocol'] == 'Telnet':
                ret = context.hosts[host].shellcmd._ssh.command(
                    cmd, timeout=10, exact_prompts='login: ').lower()
                '''
                if we can access device via telent, the prompt will show "Trying xxx(ip)...
                                                                          Connected to xxx(ip).
                                                                          Escape character is '^]'." 
                then we should use "Ctrl-d" to quit
                '''
                context.hosts[host].shellcmd._ssh.sendcontrol('d')
                assert "connected" in ret, \
                f'[TELNET shall succeed but fail]: {ret}'
            if row['protocol'] == 'SSH':
                ret = context.hosts[host].shellcmd._ssh.command(
                    cmd, timeout=10, exact_prompts='password').lower()
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                assert (context.dut_info[device]["testbed"][f"{interface.lower()}_ip"] in ret) or ("rsa key fingerprint" in ret), \
                f'[SSH shall succeed but fail]: {ret}'
        else:
            if row['protocol'] == 'HTTP':
                ret = context.hosts[host].shellcmd._ssh.command(cmd, timeout=5)
                assert (ret == None) or ("http/1.1 200 OK" not in ret.lower()) or ("http/1.1 308 permanent redirect" in ret.lower()), \
                f'[HTTP shall fail but succeed]: {ret}'
            if row['protocol'] == 'HTTPS':
                ret = context.hosts[host].shellcmd._ssh.command(cmd, timeout=5)
                assert (ret == None) or "ssl certificate problem" not in ret.lower(), \
                f'[HTTPS shall fail but succeed]: {ret}'
            if row['protocol'] == 'Telnet':
                ret = context.hosts[host].shellcmd._ssh.command(
                    cmd, timeout=5, exact_prompts='login: ')
                '''
                if we cannot access device via telent, the prompt will show "Trying xxx(ip) ..." 
                then we should use "Ctrl-c" to quit
                '''
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                assert (ret == None) or ("connected" not in ret), \
                f'ret= {ret}, [TELNET shall fail but succeed]: {ret}'
            if row['protocol'] == 'SSH':
                ret = context.hosts[host].shellcmd._ssh.command(cmd,
                                                           timeout=5,
                                                           exact_prompts=':')
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                assert (ret == None) or ("password" in ret.lower()) or ("rsa key fingerprint?" not in ret.lower()), \
                f'ret= {ret}, [SSH shall fail but succeed]: {ret}'



import time

from host import *
from behave import *
from steps import common, interface, login


@then(
    u'displayed pacakge version should be consistent with "{device}"s default version'
)
def step_impl(context, device):
    for _ in range(5):
        time.sleep(1)
        cur_versions = context.dut[device].main().show_package()
        cur_security_version, cur_mxsecurity_version = cur_versions[0].get('security'), cur_versions[0].get('mxsecurity')
        if all((cur_security_version, cur_mxsecurity_version)):
            break

    default_security_version = context.table[0]['security']
    default_mxsecurity_version = context.table[0]['mxsecurity']

    assert (default_security_version in cur_security_version) and (default_mxsecurity_version in cur_mxsecurity_version) , \
        f'returned version is inconsistant with default version: cur -> {cur_versions[0]} <-> defaut -> {context.table[0]}'


@when(u'upgrade security package {security_version} on "{device}"')
def step_impl(context, device, security_version):
    ret = context.dut[device].main().upgrade_package(tftp_ip='192.168.127.92',
                                                     pkg_type='security',
                                                     file_name=security_version)
    assert 'successfully' in ret[
        'data'], f'Upgrade security pacakge failed: {ret}'


@when(u'upgrade mxsecurity package {mxsecurity_version} on "{device}"')
def step_impl(context, device, mxsecurity_version):
    ret = context.dut[device].main().upgrade_package(
        tftp_ip='192.168.127.92',
        pkg_type='mxsecurity',
        file_name=mxsecurity_version)
    assert 'successfully' in ret[
        'data'], f'Upgrade mxsecurity pacakge failed: {ret}'


@then(
    u'displayed security package version should be consistent with "{device}"s {checked_security_version}'
)
def step_impl(context, device, checked_security_version):
    cur_versions = context.dut[device].main().show_package()
    cur_security_version = cur_versions[0]['security']

    assert (checked_security_version in cur_security_version), \
        f'returned version is inconsistant with default version: cur -> {cur_versions[0]} <-> defaut -> {context.table[0]}'


@then(
    u'displayed mxsecurity package version should be consistent with "{device}"s {checked_mxsecurity_version}'
)
def step_impl(context, device, checked_mxsecurity_version):
    cur_versions = context.dut[device].main().show_package()
    cur_mxsecurity_version = cur_versions[0]['mxsecurity']


    assert (checked_mxsecurity_version in cur_mxsecurity_version) , \
        f'returned version is inconsistant with default version: cur -> {cur_versions[0]} <-> defaut -> {context.table[0]}'



import re
import json
import time

from datetime import datetime
from session import SshSession

class UICtrlService:
    """
    This class provides methods to control the DUT UI service on a remote device through SSH connection.

        Args:
            ssh_session (SshSession): An active SSH session to the remote host.
            sudo_paswd (str): The sudo password for the remote host.
        
        Attributes:
            _ssh (SshSession): An instance of SshSession class to establish SSH connection with the remote device.
            image_name (str): The name of the image used for running the DUT UI service.
            image_tag (str): The tag of the image used for running the DUT UI service.
            container_name (str): The name of the container running the DUT UI service. 
            command_curl (str): The command used to run curl in a docker container. 
            command_dut_ui (str): The command used to run dut-ui in a docker container. 
            sudo_password (str): The password used for sudo authentication when running commands in a docker container.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self.image_name = "nbg-jenkins.moxa.com:5000/sqa/auto-test-atb-dut-service"
        self.image_tag = "v1.0.0"
        self.container_name = ""

        self.command_curl = f"docker run -it --rm {self.image_name}:{self.image_tag} curl "

    def _check_conneted(self):
        """
        Check if the SSH session is connected.

            Raises:
                ValueError: If the SSH session is not connected.
        """
        if not self._ssh.is_connected():
            raise ValueError('SSH session not connected')

    def check_env(self):
        """
        Check the environment for the given image name.

            Raises:
                ValueError: If no docker image with the given name is found. 
        """
        self._check_conneted()
        output = self._ssh.command('docker images')
        if self.image_name not in output:
            raise ValueError(f'No docker image {self.image_name}')

    def cli_init(self):
        """
        Run a docker container with the given image name and tag. 
        The container name is generated using the current datetime in ISO format.  
        """
        self.container_name = "dut_ui_"
        self.container_name += datetime.now().isoformat().replace(":", "_")[:19]
        ret = self._ssh.command(
            f"docker run -dit --name {self.container_name} {self.image_name}:{self.image_tag} /bin/bash"
        )
        print(ret)

    def cli_destroy(self):
        """
        Removes the container created for the testcase, once the testcase is finished.
        """
        self._ssh.command(f"docker rm -f {self.container_name}")
        self.container_name = ""
        time.sleep(3)

    def cli_cmd(self,
                cmd,
                detach: bool = False,
                destroy_after_cmd: bool = False):
        """
        Run a command in the CLI container.

            Args:
                cmd (str): The command to be run in the CLI container.
                detach (bool, optional): If True, run the command in detached mode. Defaults to False.
                destroy_after_cmd (bool, optional): If True, destroy the CLI container after running the command. Defaults to False.
                
            Returns:
                str: The output of the command run in the CLI container.
        """
        if self.container_name == "":
            self.cli_init()
        if detach:
            ret = self._ssh.command(
                f"docker exec -dit {self.container_name} {cmd}")
        else:
            ret = self._ssh.command(
                f"docker exec -it {self.container_name} {cmd}")

        if destroy_after_cmd:
            self.cli_destroy()

        return ret

    def web_get(self,
                uri: str = None,
                jwt: str = None,
                ip: str = "192.168.127.254",
                web_https: bool = True,
                filename: str = None):
        """
        Executes a command to get data from a web server.

            Args:
                uri (str): The URI of the web server. Required parameter.
                jwt (str): The JWT token used for authentication. Required parameter. 
                ip (str): The IP address of the web server. Defaults to "192.168.127.254". 
                web_https (bool): Flag indicating if HTTPS should be used for communication with the web server or not. Defaults to True (HTTPS).

            Returns:
                str: The response from the web server in JSON format or None if an error occurs during execution of the command. 
        """
        if uri is None:
            raise ValueError(f'uri is required.')

        if jwt is None:
            raise ValueError(f'jwt is required.')

        web_command = self.command_curl
        web_command += "-s -k -w \"\\n\" "
        web_command += f"-H \"Authorization: {jwt}\" "
        if web_https:
            web_command += f"https://{ip}/{uri} "
        else:
            web_command += f"http://{ip}/{uri} "
        web_command += '' if filename is None else f'> {filename}'
            
        ret_raw = self._ssh.command(web_command, timeout=5)

        matches = re.findall(r'{.*}', ret_raw)
        if matches:
            return re.findall(r'{.*}', ret_raw)
        else:
            return ret_raw


    def web_post(self,
                 uri: str = None,
                 data_dict: dict = None,
                 jwt: str = None,
                 ip: str = "192.168.127.254",
                 web_https: bool = True):
        """
        Send a web POST request to the given URI.

            Args:
                uri (str): The URI to send the request to. Required.
                data_dict (dict): A dictionary of data to be sent in the body of the request. Optional.
                jwt (str): The JWT token used for authentication. Required.
                ip (str): The IP address of the server to send the request to. Defaults to "192.168.127.254". Optional. 
                web_https (bool): Whether or not to use HTTPS for the request, defaults to True. Optional. 
                
            Returns: 
                str: The response from the server, or None if an error occurred during sending the request. 
        """
        if uri is None:
            raise ValueError(f'uri is required.')

        if jwt is None:
            raise ValueError(f'jwt is required.')

        web_command = self.command_curl
        web_command += "-s -k -w \"\\n\" -H \"Content-Type: application/json\" -X POST "
        web_command += f"-H \"Authorization: {jwt}\" "
        if web_https:
            web_command += f"https://{ip}/{uri} "
        else:
            web_command += f"http://{ip}/{uri} "
        if data_dict is not None:
            web_command += f"-d '{json.dumps(data_dict)}' "

        # print('web_post_command: ', web_command)
        ret_raw = self._ssh.command(web_command, timeout=5)
        # print('web_post_ret_raw: ', ret_raw)

        return re.findall(r'{.*}', ret_raw)


    def web_get_jwt(self,
                    username: str = "",
                    password: str = "",
                    ip: str = "192.168.127.254",
                    web_https: bool = True):
        """
        Retrieve JWT from web API using curl command.

            Args:
                username (str): Username for authentication. Default is an empty string.
                password (str): Password for authentication. Default is an empty string.
                ip (str): IP address of the server. Default is "192.168.127.254".
                web_https (bool): Whether to use https or not when connecting to the server. Default is True. 
                
            Returns:
                str: JWT token if successful, None otherwise. 
        """
        web_command = self.command_curl
        web_command += "-s -k -w \"\\n\" -H \"Content-Type: application/json\" -X POST "
        if web_https:
            web_command += f"https://{ip}/api/v1/auth/login "
        else:
            web_command += f"http://{ip}/api/v1/auth/login "
        web_command += f"-d '{{\"username\": \"{username}\", \"password\": \"{password}\"}}' "

        # print('web_jwt_command: ', web_command)
        ret_raw = self._ssh.command(web_command, timeout=5)
        # print('web_get_jwt_ret_raw: ', ret_raw)

        res_text = re.findall(r'{.*}', ret_raw)
        # # print('web_get_jwt_res_text: ', res_text)
        # print('res_text: ', res_text)
        res_json = json.loads(res_text[1])
        # # print('web_get_jwt_res_json: ', res_json)
        if "access_token" not in res_json:
            print("API response:", ret_raw)
            print(f'Fail to retrieve JWT from API response')
            return None

        jwt = res_json["access_token"]
        # print('jwt: ', jwt)
        return jwt


    def web_post_multipart(self,
                 uri: str = None,
                 form_name: str = None,
                 form_filename: str = None,
                 jwt: str = None,
                 ip: str = "192.168.127.254"):
        """
        Send a multipart POST request to post a file along with some parameters.

            Args:
                uri (str): The URI to send the request to. Required.
                form_name: The type of the upload file. Required
                form_filename: The name of the upload file. Required
                jwt (str): The JWT token used for authentication. Required.
                ip (str): The IP address of the server to send the request to. Defaults to "192.168.127.254". Optional. 
                
            Returns: 
                str: The response from the server, or None if an error occurred during sending the request. 
        """
        web_command = "curl -s -k -w \"\\n\" -X POST "
        web_command += f"--location 'https://{ip}/{uri}' "
        web_command += f"-H \"Authorization: {jwt}\" -H \"content-type: multipart/form-data\" "
        web_command += f"--form 'name=\"{form_name}\"' "
        web_command += f"--form 'filename=@\"{form_filename}\"'"

        print(f'=======web = {web_command}')
        ret_raw = self._ssh.command(web_command, timeout=5)
        print(f'=======ret = {ret_raw}')

        return re.findall(r'{.*}', ret_raw)



from functools import wraps

def root_access_required(func):
    @wraps(func)
    def wrapper(service_instance, *args, **kwargs):
        # Get the session from the service_instance
        session = service_instance._ssh
        # Login as root
        session.login_as_root()
        # Call the original function with the given arguments and keyword arguments
        result = func(service_instance, *args, **kwargs)
        # Logout from root
        session.logout_from_root()

        return result

    return wrapper


import time
from behave import *

@when('set mirrored port and mirror port on "{device}"')
def step_impl(context, device):
    print(context.table[0])
    src="1/"+context.host_info[context.table[0]['mirrored']]['testbed']['port']
    dst="1/"+context.host_info[context.table[0]['mirror']]['testbed']['port']

    context.dut[device].go_config().set_port_mirror(src=src,dst=dst)


@when('"{host}" start capture packet')
def step_impl(context, host):
    context.tcpdump_pid = context.hosts[host].shellcmd.tcpdump(itf=context.host_info[host]['nic'], filter_type="arp")


@when('"{host}" arp "{device}"s {interface}')
def step_impl(context, host, device, interface):
    context.arp_ip = context.dut_info[device]["testbed"][f"{interface.lower()}_ip"]
    context.hosts[host].shellcmd.arp(count=4, ip=context.arp_ip)
    time.sleep(5)

@then('"{host1}" will receive arp packet sent from "{host2}"')
def step_impl(context, host1, host2):
    ret = context.hosts[host1].shellcmd.get_tcpdumpfile(context.tcpdump_pid)
    assert f"Request who-has {context.arp_ip}" in ret, f"{host1} did not receive arp packet sent from {host2}"



from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigRstp(config.Config):
    def __init__(self, session: object, *args, **kwargs):
        """
         Initialize the instance. This is called by the : class : ` Session ` when it is created.
         
            Args:
                session: The session to use for the request / response
        """
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
         Set the default prompts for this service. This is called by __init__ and should not be called by user code
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        """
         Get input from the device. This is a no - op if the device is not connected
        """
        self._s.command_expect(f'redundancy')

    def getin(self):
        """
         Set prompts to getin. This is a shortcut for _set_default_prompts ( self. _default_prompts )
         
            Returns: 
                self for daisychaining ( self )
        """
        self._set_default_prompts()
        self._getin()
        return self
    def set_rstp_rule(self,
                      priority: str = None,
                      hello_time: str = None,
                      fwd_delay: str = None,
                      max_age: str = None,
                      ):     
        """
        Set RSTP rule. This is a function that allows to set the spanning tree of a project based on the values passed to it
        
            Args:
                priority: priority of the rule in percent
                hellotime: time to wait before sending a message
                forwarddelay: time to wait before sending a message
                maxage: maximum age of the rule in seconds ( 0 - 9 )
            
            Returns: 
                self for daisychaining ( self )
        """
        # if priority is given, set the spanning tree priority
        if priority:
            self._s.command_expect(f'spanning-tree priority {priority}')
        # if hellotime is given, set the spanning tree hellotime
        if hello_time:
            self._s.command_expect(f'spanning-tree hello-time {hello_time}')
        # if forwarddelay is given, set the spanning tree forwarddelay
        if fwd_delay:
            self._s.command_expect(f'spanning-tree forward-delay {fwd_delay}')
        # if maxage is given, set the spanning tree maxage
        if max_age:
            self._s.command_expect(f'spanning-tree max-age {max_age}')
        return self



from mdc.router.cli.rp_base.lg_router.edr800 import config
from mdc.router.cli.rp_base.lg_router.edr800.cli import PROMPT_CONFIG

class ConfigIfLan(config.Config):
    """
    Class for configuring LAN interface.

        Args:
            session (Session): Session object to use for communication with the device.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (tuple): Tuple of positional arguments.
            _kwargs (dict): Dictionary of keyword arguments. 
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        self._s.command_expect('interface lan')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def name(self, name):
        self._s.command_expect(f'name {name}')
        return self

    def set_lan_interface(self,
                            name=None,
                            ip=None,
                            mask=None,
                            bind_vlan=None,
                            mac_address=None):
        if name:
            self._s.command_expect(f'name {name}')
        if ip and mask:
            self._s.command_expect(f'ip address static {ip} {mask}')
        if bind_vlan:
            self._s.command_expect(f'bind vlan {bind_vlan}')
        if mac_address:
            self._s.command_expect(f'mac-address {mac_address}')
        return self

    def ip_address_static(self, ip, mask):
        self._s.command_expect(f'ip address static {ip} {mask}')
        return self



from scapy.all import *
raw_data = bytes.fromhex("00" * {padding_n})
# 建立VRRP封包
vrrp_packet = Ether(src="{smac}", dst="{dmac}") \
            / IP(src="{sip}", dst="{dip}", ttl=255) \
            / VRRPv3(version={version}, priority={priority}) \
            / Padding(load=raw_data)
ip_address = "{ip_address}"  # 要添加的IP地址
vrrp_packet[VRRPv3].addrlist = ip_address.split(",")

# 送出VRRP封包
sendp(vrrp_packet, iface="{iface}")


import time

@when(u'run ospf service on "{host}"')
def step_impl(context, host):
    intf = context.host_info[host]["nic"]
    area_id = context.table[0]["area_id"]
    priority = int(context.table[0]["priority"])
    hello_interval = int(context.table[0]["hello_interval"])
    dead_interval = int(context.table[0]["dead_interval"])
    router_id = context.host_info[host]["testbed"]["cur_host"]
    context.hosts[host].bird.set_ospf_config(router_id = router_id,
                                             intf = intf, 
                                             area_id = area_id,
                                             priority = priority,
                                             hello_interval = hello_interval, 
                                             dead_interval = dead_interval)
    context.hosts[host].bird.start()
    context.ospf_host = host

@when(u'set ospf config on "{device}" {intf}')
def step_impl(context, device, intf):
    area_id = context.table[0]["area_id"]
    priority = int(context.table[0]["priority"])
    hello_interval = int(context.table[0]["hello_interval"])
    dead_interval = int(context.table[0]["dead_interval"])
    router_id = context.dut_info[device]["testbed"]["wan_ip"]
    redistribute = None if context.table[0]["redistribute"] == "disable" \
        else context.table[0]["redistribute"]
    context.dut[device].go_config_ospf(router_id).set_ospf_area(area_id=area_id)
    context.dut[device].go_config_ospf(router_id).set_ospf_intf(intf=intf,
                                                                area_id=area_id,
                                                                priority=priority,
                                                                hello_interval=hello_interval,
                                                                dead_interval=dead_interval)
    context.dut[device].go_config_ospf(router_id).set_ospf_redistribute(redistribute)
    
@then(u'ospf should be "{state}" on "{device}" {intf}')
def step_impl(context, state, device, intf):
    ret = context.dut[device].main().show_ip_ospf_interface()
    flag = False
    for _ in range(12):
        if ret[0]["state"] == state:
            flag = True
            break
        ret = context.dut[device].main().show_ip_ospf_interface()
    context.hosts[context.ospf_host].bird.clean_up()
    assert flag, f'ospf act as wrong state on {device}: expect {state} but actual {ret["state"]}'

@then(u'ospf neighbor {action} contain "{host}" on "{device}"')
def step_impl(context, host, action, device):
    ret = context.dut[device].main().show_ip_ospf_neighbor()
    flag = False
    for _ in range(12):
        try:
            ret = context.dut[device].main().show_ip_ospf_neighbor()
            if context.host_info[host]["testbed"]["cur_host"] in ret[0]["neighbor_id"] \
                and "2-Way" not in ret[0]["state"]:
                flag = True
                break
        except:
            print(f"no any ospf neighbor on {device}")
    context.hosts[context.ospf_host].bird.clean_up()
    assert flag if action == "should" else not flag, \
    f'ospf neighbor {action} have {context.host_info[host]["testbed"]["cur_host"]} on {device} but {ret}'

@then(u'ospf database {action} have ospf information from "{device}"s {intf}')
def step_impl(context, action, device, intf):
    import ipaddress
    def get_subnet(ip, mask):
        # Convert IP and mask to IPv4Address and IPv4Network objects
        ip_obj = ipaddress.IPv4Address(ip)
        network_obj = ipaddress.IPv4Network(f"{ip}/{mask}", strict=False)
        
        # Return the network address of the subnet
        return str(network_obj.network_address)
    def mask_to_cidr(mask):
        # Split the mask into octets and convert each octet to binary
        binary_mask = ''.join([bin(int(octet))[2:].zfill(8) for octet in mask.split('.')])
        
        # Count the number of '1' bits in the binary representation
        return binary_mask.count('1')
    ret = context.dut[device].main().show_ip_ospf_database()
    subnet = get_subnet(context.dut_info[device]["testbed"][f"{intf.lower()}_ip"], 
                        context.dut_info[device]["testbed"][f"{intf.lower()}_netmask"])
    mask = mask_to_cidr(context.dut_info[device]["testbed"][f"{intf.lower()}_netmask"])
    flag = False
    print(subnet, mask, context.dut_info[device]["testbed"]["wan_ip"] )
    for _ in range(12):
        try:
            ret = context.dut[device].main().show_ip_ospf_database()
            if subnet in ret[0]["link_state_id"] and \
               context.dut_info[device]["testbed"]["wan_ip"] in ret[0]["adv_router"] and \
               f'{subnet}/{mask}' in ret[0]["router"] and \
                "AS-external" in ret[0]["ls_type"]:
                flag = True
                break
        except:
            print(f"no any external route on ospf database")
    context.hosts[context.ospf_host].bird.clean_up()
    assert flag if action == "should" else not flag, \
        f'ospf database should have external route but {ret}'
    
@then(u'"{host}" should receive Type 3 Summary LSAs')
def step_impl(context, host):
    context.hosts[host].tshark.capture_by_number(interface='eth1',
                                                 number = 5,
                                                 timeout = 60,
                                                 capture_filter = f'ip proto 89 and src host 192.168.128.254 and ip[21] = 4',
                                                 additional_options = "-O ospf")
    context.hosts[host].tshark.stop()
    rcv_pkt = context.hosts[host].tshark.retrieve()
    print(rcv_pkt)

    assert "LSA-type 3 (Summary-LSA (IP network))" in rcv_pkt, \
    f"packets with [LSA-type 3] are not existed"


from host import *
from behave import *
import time


@when(u'set snmp version to {snmp_version} on "{device}"')
@given(u'set snmp version to {snmp_version} on "{device}"')
def step_impl(context, snmp_version, device):
    context.cfg = context.dut[device].go_config()
    context.cfg.Snmp(context.cfg.get_session()).set_snmp_version(snmp_version)


## only SNMPv3
@when(u'set snmp user authentication and privacy method on "{device}"')
def step_impl(context, device):
    context.user = context.table[0]['User']
    context.security = context.table[0]['Security level']
    context.auth_type = context.table[0]['Auth Type'].lower()
    if context.table[0]['Priv Method'].lower() == 'none':
        context.priv_method = None
    else:
        context.priv_method = context.table[0]['Priv Method'].lower()
    context.priv_key = context.table[0]['Priv Key']

    context.cfg.Snmp(context.cfg.get_session()).set_snmp_user(
        user=context.user,
        priv_key=context.priv_key,
        auth_type=context.auth_type,
        priv_method=context.priv_method)


## only SNMPv3
@when(u'set password for the following account on the "{device}"')
@given(u'set password for the following account on the "{device}"')
def step_impl(context, device):

    if context.auth_type != 'no-auth':

        context.auth_key = context.table[0]['Password']

        ## default.yaml user password
        context.dut_info[device]['credential']['password'] = context.auth_key
        context.cfg.set_login_account(action='modify',
                                      username=context.table[0]['User'],
                                      password=context.auth_key)
    else:
        context.auth_key = None


## only SNMPv3
@when(u'set snmp engineid to "{engine_id}"')
def step_impl(context, engine_id):
    context.cfg.Snmp(
        context.cfg.get_session()).set_engine_id(engine_id=engine_id)


@then(u'the following rule should be on "{device}"s snmp setting table')
def step_impl(context, device):
    flag = False
    snmp_setting = context.dut[device].main().show_snmp()

    for entry in snmp_setting:
        if entry['version'] == context.table[0]['Version'] and \
            entry['engine_id'] == context.table[0]['Engine ID'] and \
            entry['admin_auth_status'] == context.table[0]['Auth status'] and \
            len(entry['admin_auth_key']) == len(context.table[0]['Auth passwd']) :
            flag = True
            break

    assert flag, set_to_default_passwd(
        context,
        f"No match found -> expect: {context.table[0]}, actual: {snmp_setting}")


@when(u'get the sysname from "{device}"')
def step_impl(context, device):
    res = context.dut[device].main().show_system()
    time.sleep(5)
    context.sysname = res[0]['system_name']


@then(
    u'"{host}" {can_or_cannot} get "{device}"s sysname by SNMP version {version}'
)
def step_impl(context, host, can_or_cannot, device, version):

    if version == '1' or version == '2c':
        ret = context.hosts[host].snmp.get_snmp(
            version=version,
            community='public',
            agent_id=context.dut_info[device]['testbed']['lan_ip'],
            oid='1.3.6.1.2.1.1.5.0')

    if version == '3':
        ret = context.hosts[host].snmp.get_snmp(
            version=version,
            username=context.user,
            security=context.security,
            auth_type=context.auth_type,
            auth_key=context.auth_key,
            priv_method=context.priv_method,
            priv_key=context.priv_key,
            agent_id=context.dut_info[device]['testbed']['lan_ip'],
            oid='1.3.6.1.2.1.1.5.0')

    if can_or_cannot == 'can':
        assert context.sysname in ret, set_to_default_passwd(
            context,
            f'Expected to receive SNMP message, but did not receive. actual: {ret}'
        )

    if can_or_cannot == 'can not':
        assert context.sysname not in ret, set_to_default_passwd(
            context,
            f'Expected not to receive SNMP message, but received. return: {ret}'
        )


@given(
    u'"{host}" send snmp set request to set "{device}"s sysnem as "{new_sysname}" by SNMP version {version}'
)
def step_impl(context, host, device, version, new_sysname):

    if version == '1' or version == '2c':
        ret = context.hosts[host].snmp.set_snmp(
            version=version,
            community='private',
            agent_id=context.dut_info[device]['testbed']['lan_ip'],
            oid='1.3.6.1.2.1.1.5.0',
            new_info=new_sysname)

    if version == '3':
        ret = context.hosts[host].snmp.set_snmp(
            version=version,
            username=context.user,
            security=context.security,
            auth_type=context.auth_type,
            auth_key=context.auth_key,
            priv_method=context.priv_method,
            priv_key=context.priv_key,
            agent_id=context.dut_info[device]['testbed']['lan_ip'],
            oid='1.3.6.1.2.1.1.5.0',
            new_info=new_sysname)


@then(u'"{device}"s sysname {can_or_not} be set to "{new_sysname}"')
def step_impl(context, device, can_or_not, new_sysname):
    time.sleep(5)
    res = context.dut[device].main().show_system()
    current_sysname = res[0]['system_name']

    if can_or_not == 'can':
        assert current_sysname == new_sysname, set_to_default_passwd(
            context,
            f'No match found, except -> {new_sysname}; actual -> {current_sysname}'
        )
    else:
        assert current_sysname != new_sysname, set_to_default_passwd(
            context,
            f"Sysname should not be same, but it's same -> {new_sysname}; actual -> {current_sysname}"
        )


@given(u'set password and sysname to default on the "{device}"')
def step_impl(context, device):
    set_to_default_passwd(context, message=None)
    context.cfg.set_hostname(context.sysname)
    res = context.dut[device].main().show_system()


def set_to_default_passwd(context, message):
    try:
        cfg = context.dut['DUT'].go_config()
        cfg.set_login_account(action='modify',
                              username="admin",
                              password="moxa")
    except:
        print('The sysname has not been changed by this test case.')

    if message:
        print(message)


@given(u'set snmp trap/inform trap mode to {mode} on "{device}"')
def step_impl(context, mode, device):

    if 'v3' in mode:
        user = context.table[0]['User']
        auth_type = context.table[0]['Auth Type'].lower()
        passwd = context.table[0]['Password']
        priv_key = context.table[0]['Priv Key'] if \
            context.table[0]['Priv Key'] else None
        context.cfg.Snmp(context.cfg.get_session()).set_snmp_trap_inform_v3(
            trap_mode=mode,
            user=user,
            auth_type=auth_type,
            passwd=passwd,
            priv_key=priv_key)
    # SNMPv2-inform
    elif 'inform' in mode and '2c' in mode:
        mode = 'inform'
        retry = context.table[0]['Inform Retry']
        timeout = context.table[0]['Inform Timeout']
        context.cfg.Snmp(context.cfg.get_session()).set_snmp_trap_mode(
            trap_mode=mode, retry=retry, timeout=timeout)
    # SNMPv1, v2 trap
    else:
        context.cfg.Snmp(
            context.cfg.get_session()).set_snmp_trap_mode(trap_mode=mode)


@given(u'set snmp trap/inform host to receive snmp notification on "{device}"')
def step_impl(context, device):
    host_ip = context.table[0]['Host IP']
    community = context.table[0]['Community']

    context.cfg.Snmp(context.cfg.get_session()).set_snmp_trap_host(
        host_ip, community)


@when('starts tshark sniffer on "{host}" for snmp packet')
def step_impl(context, host):
    context.hosts[host].tshark.start(interface=context.host_info[host]['nic'],
                                     display_filter="snmp")
    time.sleep(5)


@when(
    'starts tshark sniffer and decrypt message on "{host}" for snmp packet {with_without} verbose'
)
def step_impl(context, host, with_without):
    uat = context.table[0]['UAT']
    username = context.table[0]['username']
    auth_type = context.table[0]['Auth Type']
    passwd = context.table[0]['Password']
    encry_method = context.table[0]['Encrypt Method']
    encry_key = context.table[0]['Encrypt Key']
    verbose = True if with_without == 'with' else False

    context.hosts[host].tshark.start(
        interface=context.host_info[host]['nic'],
        display_filter="snmp",
        verbose=verbose,
        option=
        f'uat:{uat},\"{username}\",\"{auth_type}\",\"{passwd}\",\"{encry_method}\",\"{encry_key}\"'
    )
    time.sleep(5)


@given(
    u'starts snmptrapd tool on "{host}" to report SNMPv3 inform to get informRequest'
)
def step_impl(context, host):
    context.hosts[host].snmp.start_snmptrapd()


@then(
    u'"{host}" should receive {pkt_num} snmp {pkt_name} packet from "{device}"')
def step_impl(context, host, pkt_num, pkt_name, device):
    time.sleep(10)
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve().split('\n')

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    pkt_count = 0

    for line in ret:
        if 'SNMP' in line and pkt_name in line:
            pkt_count += 1

    assert pkt_count >= int(
        pkt_num
    ), f'except to receive {pkt_num} packet(s) but receive {pkt_count}.'


@then(
    u'"{host}" should receive {pkt_num} {pkt_name} snmp packet with the following info from "{device}"'
)
def step_impl(context, host, pkt_num, pkt_name, device):
    time.sleep(10)
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve().split('\n')

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    pkt_count = 0
    info = context.table[0]['info']

    for line in ret:
        if 'SNMP' in line and pkt_name in line and info in line:
            pkt_count += 1

    assert pkt_count >= int(
        pkt_num
    ), f'except to receive {pkt_num} snmp packet(s) but receive {pkt_count}.'


@then(
    u'"{host}" should receive snmp packet with engine id that is same as "{device}"s snmp table'
)
def step_impl(context, host, device):
    snmp_setting = context.dut[device].main().show_snmp()
    except_engine_id = snmp_setting[0]['engine_id']
    time.sleep(10)
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve().split('\n')

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    flag = False
    actual_engine_id = []

    for line in ret:
        if 'contextEngineID' in line:
            actual_engine_id.append(line.strip())
            if except_engine_id in line:
                flag = True

    assert flag, f'The engine id of snmp should be {except_engine_id} but {actual_engine_id}'



import os
import re
import json

from datetime import datetime
from session import SshSession


class CaService:
    """
    Class for handling certificate authority related operations.

        Args:
            ssh_session (SshSession): SSH session object.
            sudo_paswd (str): sudo password.
        
         Attributes:
            _ssh (SshSession): An SSH session object.

        Tip:
            1. Create CA certificate
            2. Create Server certificate signing request (CSR)
            3. Sign the Server Request using CA certificate
    """
    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._sudo_pwd = sudo_paswd

    def check_env(self):
        if not self._ssh.is_connected():
            raise ValueError('SSH session not connected')

        ret = self._ssh.command('openssl version', timeout=5)
        assert 'not found' not in ret, \
                f'Command "openssl" not found'


    def prepare_env(self):
        self._ssh.command('mkdir cert', timeout=5)
    

    def generate_root_ca(self, 
                         rsa: str = "4096"):
        """
        Generate a self-signed X.509 certificate (also known as a self-signed certificate).

            Tip:
                openssl req: Certificate requests and related operations.
                -x509: Specifies that a self-signed certificate will be created instead of a certificate signing request (CSR).
                -newkey rsa:4096: Generates a new RSA private key with a key size of 4096 bits.
                -keyout root/root.pem: Output file where the private key will be stored.
                -out root/root.crt: Specifies the output file where the self-signed certificate will be stored.
                -days 3650: Setup the validity period of the certificate to 3650 days.
                -nodes: Specifies that the private key should not be encrypted with a passphrase. 
                        (It allows for automated processes since no password will be required when using the key.)
                -subj: Setup the subject (or distinguished name) of the certificate. 
                        C=Country code; ST=State or province name; L=Locality or city name; 
                        O=Organization name; OU=Organizational unit name; CN=Common Name
        """ 
        cmd = 'openssl req -x509 '
        cmd += f'-newkey rsa:{rsa} '
        cmd += '-keyout cert/root.pem -out cert/root.crt '
        cmd += '-days 3650 '
        cmd += '-nodes '
        cmd += '-subj "/C=TW/ST=Taipei/L=Taipei/O=Dp3/OU=Sec5/CN=Dp3_Sec5_localhost"'
        
        self._ssh.command(cmd, timeout=5)

    
    def generate_server_csr(self, 
                            rsa: str = "2048",
                            cn: str = "192.168.127.254"):
        """
        Generate a certificate signing request (CSR) for a server.

            Args:
                rsa (str): key size
                cn (str): Common Name (should be host name)
        """
        file_name = f'server_{datetime.now().isoformat().replace(":", "_")[:19]}'
        
        cmd = 'openssl req '
        cmd += f'-newkey rsa:{rsa} '
        cmd += f'-keyout cert/{file_name}.pem -out cert/{file_name}.csr '
        cmd += '-days 3650 '
        cmd += '-nodes '
        cmd += f'-subj "/CN={cn}"'

        self._ssh.command(cmd, timeout=5)

    
    def sign_cert(self, 
                 server_csr: str = "192.168.127.254.csr",
                 root_cert: str = "root.crt",
                 root_penkey: str = "root.pem",
                 server_cert: str = "test.crt"):
        """
        Generating a signed certificate (server_cert.crt),
            using the private key of the root certificate (CA) to sign the certificate request (CSR), 

            Args:
                server_csr (str): The input file for credential requests.
                root_cert (str): The root certificate (CA) file.
                root_penkey (str): The private key file of the root certificate (CA).
                server_cert (str): The output file for signed certificates.
        """
        cmd = 'openssl x509 -req '
        cmd += f'-in {server_csr} -CA cert/{root_cert} -CAkey cert/{root_penkey} -CAcreateserial '
        cmd += f'-out cert/{server_cert} '
        cmd += '-days 3650'

        ret = self._ssh.command(cmd, timeout=5)

    
    def check_secure_connection(self, 
                                cert: str = "cert/root.crt",
                                pem_key: str = "cert/root.pem",
                                ip: str = "192.168.127.254"):
        """
        Check if a secure connection can be established using the generated certificate.

            Args:
                cert (str): Specifies the path to a CA certificate used to verify certificates presented by the server.
                pem_key (str): Specifies the path to the client's private key file.
                ip (str): Target ip. Defaults to "192.168.127.254".
        """
        cmd = f'curl -vvI --cacert {cert} --key {pem_key} https://{ip}'
        ret = self._ssh.command(cmd, timeout=5)

        if 'SSL certificate verify ok.' not in ret:
            return False

        return True


    def clean_up(self, 
                 username: str = 'autotwo'):
        """
        Clean up the environment.
        """
        self._ssh.command(f'ssh-keygen -f "/home/{username}/.ssh/known_hosts" -R "192.168.127.254"', timeout=2)
        self._ssh.command('ssh admin@192.168.127.254')
        self._ssh.sendcontrol('C')

    
    def rm_file(self):
        """
        Remove generated files.
        """
        self._ssh.command("rm -r cert *.csr", timeout=5)


from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigNat(config.Config):
    """
    Class for configuring NAT service.

        Args:
            session (object): The session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        """
        Get into the interface lan.
        """
        # default apply the first element as parameter
        index = self._args[0] if len(self._args) != 0 else None
        if index is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'ip nat {index}')

    def getin(self):
        """
        Set the default prompts and get into the interface wan.

            Returns: 
                self: The instance of the object. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def original(self, in_iface: str, osi: str, osp: str, odi: str,
                    odp: str):
        """
        Set original parameters of the NAT rule.

            Args:
                in_iface (str): The commining interface.
                osi (str): The original source ip.
                osp (str): The original port.
                odi (str): The original destination ip.
                odp (str): The original destination port.
                
            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(
            f'original in-iface {in_iface} src-ip {osi} src-port {osp} dst-ip {odi} dst-port {odp}'
        )
        return self

    def translated(self, out_iface: str, tsi: str, tsp: str, tdi: str,
                    tdp: str):
        """
        Set translated parameters of the NAT rule.

            Args:
                out_iface (str): The outgoing interface.
                tsi (str): The translated source ip.
                tsp (str): The translated port.
                tdi (str): The translated destination ip.
                tdp (str): The translated destination port.
                
            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(
            f'translated out-iface {out_iface} src-ip {tsi} src-port {tsp} dst-ip {tdi} dst-port {tdp}'
        )
        return self

    def enable(self):
        """
        Enable the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'enable')
        return self

    def protocol(self, protocol):
        """
        Set protocol of the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'protocol {protocol}')
        return self

    def mode(self, mode: str):
        """
        Set mode of the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'mode {mode}')
        return self

    def desc(self, desc):
        """
        Set desc of the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'desc {desc}')
        return self

    def nat_loopback(self):
        """
        Enable nat_loopback of the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'nat-loopback')
        return self

    def double_nat(self):
        """
        Enable double_nat of the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'double-nat')
        return self

    # For debug
    def show(self):
        """
        Show the configured NAT rule.

            Returns:
                self: The instance of the object. 
        """
        ret = self._s.command_expect(f'show')
        print(ret['data'])
        return self

    def exit(self):
        """
        Exit configure mode of the NAT rule.

            Returns:
                self: The instance of the object. 
        """
        ret = self._s.command_expect('exit',
                                        prompts=[PROMPT_CONFIG],
                                        timeout=2)
        if ret['data'].find("You are setting") != -1:
            raise ValueError(f'Fail to set NAT rule\n{ret["data"]}')
        return self



import time

from behave import *
from steps.login import loginToDUT
from steps import vpn

@when('create vlan with id "{vlan_id}" on "{device}"')
def step_impl(context, vlan_id, device):
    context.dut[device].go_config().set_vlan_id(vlan_id)


@when('delete vlan with id "{vlan_id}" on "{device}"')
def step_impl(context, vlan_id, device):
    context.dut[device].go_config().set_vlan_id(vlan_id, delete=True)


@when('enable file encryption with signature information as "{info}" and key string as "{key_str}" on "{device}"')
def step_impl(context, info, key_str, device):
    info = {"Encrypt all information":"all",
            "Encrypt sensitive information only":"sensitive"}[info]
    context.dut[device].main().set_config_file(dig_signature="enable", data_encrypt=info, encrypt_pwd=key_str)


@when('disable file encryption on "{device}"')
def step_impl(context, device):
    context.dut[device].main().set_config_file(dig_signature="disable")


@then('import configuration file to "{device}" from {server} failed')
def step_impl(context, device, server):
    ret = context.dut[device].main().import_config(tftp_ip=context.host_info["HOST_EXECUTOR"]["testbed"]["cur_host"])
    context.dut[device].close()
    assert "Config file import failed" in ret['data'] or "Configuration Upgrade Fail" in ret['data'], \
        f'Configuration Import Successfully: {ret["data"]}'
    loginToDUT(context, device)


@then('import configuration file to "{device}" from {server} successfully')
def step_impl(context, device, server):
    ret = context.dut[device].main().import_config(
        tftp_ip=context.host_info["HOST_EXECUTOR"]["testbed"]["cur_host"])
    context.dut[device].close()
    assert ret['matched'] and '^Parse error' not in ret[
        'data'], f'Configuration Import Fail: {ret["data"]}'
    loginToDUT(context, device)
    time.sleep(10)


@then('vlan id "{vlan_id}" is not on "{device}"')
def step_impl(context, vlan_id, device):
    ret = context.dut[device].main().show_vlan()
    for i in ret:
        if i['vid']==vlan_id:
            assert False, f'vlan id {vlan_id} is on {device}' 


@then('vlan id "{vlan_id}" is on "{device}"')
def step_impl(context, vlan_id, device):
    ret = context.dut[device].main().show_vlan()
    flag = 0
    for i in ret:
        if i['vid']==vlan_id:
            flag = 1
            break
    assert flag==1, f'vlan id {vlan_id} is not on {device}' 


@when('create NTP Authentication Keys on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_ntp_auth(context.table[0]['key_id'], context.table[0]['key_type'], context.table[0]['key'])


@when('enable PPTP Dialup on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config_if_wan().set_pptp(context.table[0]['ip'], context.table[0]['username'], context.table[0]['password'])


@when('create account for L2TP on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_l2tp_user(context.table[0]['username'], context.table[0]['password'])


@when('set email account on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_email(context.table[0]['username'], context.table[0]['password'])


@when('configure Dynamic DNS on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_ddns(name=context.table[0]['username'], pwd=context.table[0]['password'])


@when('set IEEE 802.1X radius 1st-server on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_dot1x_radius_server(server_idx=1, key=context.table[0]['share_key'])


@when('set IEEE 802.1X Local Database on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_dot1x_local_db(context.table[0]['username'], context.table[0]['password'])


@when('set RADIUS server on "{device}"')
def step_impl(context, device):
    """
     RADIUS authentication server index must be 1~2.
        1: Primary RADIUS authentication server
        2: Backup RADIUS authentication server
    """
    for row in context.table:
        context.dut[device].go_config().set_radius_server(server_idx=row['idx'], 
                                                          ip=row['ip'], 
                                                          port=row['port'], 
                                                          key=row['share_key'])


@when('add OSPF area on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config_ospf(context.table[0]['router_id']).set_ospf_area(context.table[0]['area_id'])


@when('create interface for ospf on "{device}"')
def step_impl(context, device):
    if context.table[0]['interface'] == "LAN":
        if context.table[0]['auth_type'] == "simple":
            context.dut[device].go_config_if_lan().set_ospf(area_id=context.table[0]['area_id'], 
                                                        auth={'type':context.table[0]['auth_type'],
                                                              'key':context.table[0]['auth_key']})
        elif context.table[0]['auth_type'] == "md5":
            context.dut[device].go_config_if_lan().set_ospf(area_id=context.table[0]['area_id'], 
                                                        auth={'type':context.table[0]['auth_type'],
                                                              'key':context.table[0]['auth_key'],
                                                              'key_id':context.table[0]['auth_key_id']})


@then('the following sensitive information in running config on "{device}" is encrypted')
def step_impl(context, device):
    context.running_config = context.dut[device].main().get_running_config(context.dut_info[device]["session"]["con_type"]).replace("\r", "").split('\n')
    for i in range(len(context.running_config)):
        for j in context.table:
            if j['information'] in context.running_config[i]:
                ret = context.running_config[i].split(' ')[-1]
                if len(ret)>50 and ret!=j['original_value']:
                    print(f"{j['information']}:{ret} ok pass!")                    
                    assert True
                else:
                    assert False, f"{j['information']}:{ret} not encrypted!"
                break


import os
import time
import json

from host import *
from behave import *
from steps import login, interface, common
from host import HostFactory
from session import SshSession
from datetime import datetime


@Given('"{host}" login to "{device}" web console from {intf} port to get jwt')
@When('"{host}" login to "{device}" web console from {intf} port')
def step_impl(context, host, device, intf):
    context.dut_jwt = {}
    web_jwt = context.hosts[host].dut_ui.web_get_jwt( 
        username=context.dut_info[device]['credential']['username'], 
        password=context.dut_info[device]['credential']['password'])
    context.dut_jwt[device] = web_jwt


@Then('"{host}" logout to "{device}" web console from {intf} port')
@When('"{host}" logout to "{device}" web console from {intf} port')
def step_impl(context, host, device, intf):
    # Session Clean Up
    res = context.hosts[host].dut_ui.web_post(
        uri="api/v1/auth/logout",
        jwt=context.dut_jwt[device])
    context.dut_jwt[device] = None
    
    res_str = str(res)
    assert 'logout success' in res_str, \
            f'response should be "logout success", but {res_str}'


@Given('"{host}" connect "{device}" to generate RSA Key')
def step_impl(context, host, device):
    if context.table and "rsakey_name" in context.table.headings:
        rsakey_name = context.table[0]["rsakey_name"]
    else:
        rsakey_name = "MoxaRsaKey"
    if context.table and "private_key_size" in context.table.headings:
        private_key_size = int(context.table[0]["private_key_size"])
    else:
        private_key_size = 1024

    assert private_key_size in [1024, 2048], \
            f'private_key_size should be 1024 or 2048 but {private_key_size}'

    res = context.hosts[host].dut_ui.web_post(
        uri="api/v1/auth/rsaKeyGen",
        data_dict={"rsakey_name": rsakey_name, "private_key": private_key_size},
        jwt=context.dut_jwt[device])

    res_str = str(res)
    assert 'RSA key generate success.' in res_str, \
            f'response should be "RSA key generate success.", but {res_str}'


@Given('"{host}" connect "{device}" to generate CSR')
def step_impl(context, host, device):
    if context.table and "private_key" in context.table.headings:
        private_key = context.table[0]["private_key"]       # private_key Naming Role: f'{rsakey_name}-{private_key}'
    else:
        private_key = "MoxaRsaKey-1024"
    
    country_name = context.table[0]["country_name"]
    locality_name = context.table[0]["locality_name"]
    org_name = context.table[0]["org_name"]
    org_unit_name =context.table[0]["org_unit_name"]
    email_addr =context.table[0]["email_addr"]
    
    res = context.hosts[host].dut_ui.web_post(
        uri="api/v1/auth/csrGen",
        data_dict={"privateKey": private_key,
                    "ct": country_name,
                    "local": locality_name,
                    "org": org_name,
                    "orgUnit": org_unit_name,
                    "cn": "192.168.127.254",  # file name
                    "ea": email_addr,
                    "subname": "192.168.127.254"},
        jwt=context.dut_jwt[device])
    
    res_str = str(res)
    assert 'CSR generate success.' in res_str, \
            f'response should be "CSR generate success.", but {res_str}'


@Given('"{host}" connect "{device}" to download CSR')
def step_impl(context, host, device):
    context.hosts[host].dut_ui.web_get(
        uri="csr/192.168.127.254.csr",
        jwt=context.dut_jwt[device],
        filename='192.168.127.254.csr')


@Given('"{host}" generate signed certificate base on CSR')
def step_impl(context, host):
    context.hosts[host].ca.prepare_env()
    context.hosts[host].ca.generate_root_ca()
    context.hosts[host].ca.sign_cert()


@Given('"{host}" connect "{device}" to import Certificate from CSR')
def step_impl(context, host, device):
    if context.table and "cert_name" in context.table.headings:
        cert_name = context.table[0]["cert_name"]
    else:
        cert_name = "Moxa"
    
    uri = 'api/v1/auth/cerMgmtUpload?'
    uri += 'cer_file=test.crt&'
    uri += 'mgmt_mode=1&'
    uri += f'cer_name={cert_name}&'
    uri += 'csr_file=192.168.127.254.csr'

    res = context.hosts[host].dut_ui.web_post_multipart(
        uri=uri,
        form_name="cer_file",           # file type
        form_filename="cert/test.crt",  # file name and path
        jwt=context.dut_jwt[device])

    res_str = str(res)
    assert 'Upload success.' in res_str, \
            f'response should be "Upload success.", but {res_str}'


@When ('"{device}" uses auto-generated default certificate')
def step_impl(context, device):
    host = "HOST"
    res = context.hosts[host].dut_ui.web_get(
        uri="api/v1/setting/data/SRV_AUTH_CERT",
        jwt=context.dut_jwt[device])
    
    res_str = str(res)
    assert '"selsslmode":0' in res_str, \
            f'DUT should use auto-generated default certificate("selsslmode":0), but {res_str}'
    


@When('"{device}" uses imported certificate from Local Certificate Database')
def step_impl(context, device):
    host = "HOST"
    if context.table and "cert_name" in context.table.headings:
        cert_name = context.table[0]["cert_name"]
    else:
        cert_name = "Moxa"    
    # private_key Naming Role: f'{rsakey_name}-{private_key}'
    if context.table and "private_key" in context.table.headings:
        private_key = context.table[0]["private_key"]
    else:
        private_key = "MoxaRsaKey-1024"

    pemkey = "/mnt/log/rsakey_file/" + private_key

    res = context.hosts[host].dut_ui.web_post(
        uri="api/v1/setting/data/?SRV=SRV_AUTH_CERT",
        data_dict={"SRV_AUTH_CERT": 
                        {"selsslmode": 1,
                         "selpem": cert_name,
                         "pemkey": pemkey, 
                         "auth_cert_reGen": 0}},
        jwt=context.dut_jwt[device])

    res_str = str(res)
    assert '"success": "true"' in res_str, \
            f'response should include "success": "true", but {res_str}'


@then(u'"{host}" {can_or_cannot} establish a secure connection to "{device}"')
@then(u'check if "{host}" {can_or_cannot} establish a secure connection to "{device}"')
def step_impl(context, host, device, can_or_cannot):
    flag = {'can': True, 'cannot': False}[can_or_cannot]
    
    if flag:
        try:
            for _ in range(10):
                ret = context.hosts[host].ca.check_secure_connection()
                if ret:
                    break
                time.sleep(10)
            assert ret , f'{host} unable to establish a secure connection to {device}.'
        finally:
            # clean up
            context.dut[device].main().reload_factory_default_no_cert()
            time.sleep(50)
            context.hosts[host].ca.rm_file()
            for host_clean in context.host_info:
                info = context.host_info[host_clean]
                username = context.host_info[host_clean]['session']['credential']['username']

                print(f'\n=======> start {host_clean} clean up\n')
                context.hosts[host_clean] = HostFactory.create_atb(info)
                context.hosts[host_clean].login()
                context.hosts[host_clean].ca.clean_up(username)
                print(f'\n<======= End of {host_clean} clean up\n')
    else:
        ret = context.hosts[host].ca.check_secure_connection()
        assert not ret , \
             f'Expect {host} cannot establish a secure connection to {device} but success.'

    


from scapy.all import *
raw_data = bytes.fromhex("00" * {padding_n})
ether = Ether(src="{smac}", dst="{dmac}")

# 建立LLC封包
llc = LLC(dsap=0x42, ssap=0x42, ctrl=3)

# 建立Spanning Tree Protocol (STP)封包
stp = STP(proto=0, version=2, bpdutype=2, bpduflags=62,
          rootid=32768, rootmac="{smac}",
          pathcost=int({path_cost}), bridgeid=int({brid}), bridgemac="{smac}",
          portid=32770, age=0.0, maxage=int({max_age}), hellotime=int({hello_time}), fwddelay=int({fwd_delay}))

# 將封包組合在一起
rstp_packet = ether / llc / stp / Padding(load=raw_data)

# 送出VRRP封包
sendp(rstp_packet, iface="{iface}")


import re
import sys
import time

from host import *
from behave import *
from steps import common, interface, login
from mdc.router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000


@given(
    u'set a dos rule to deny {dos_type} and enable logging flash on "{device}"')
def step_impl(context, dos_type, device):
    # dos icmp-death 1000
    # no dos icmp-death
    # dos syn-flood 1000
    # no dos syn-flood
    # dos arp-flood 1000
    # no dos arp-flood
    dos_type = dos_type.lower().replace(' ', '-')
    context.dut[device].go_config(). \
    command(f'dos {dos_type}'). \
    command(f'logging dos'). \
    command(f'logging dos flash')
    dos_type = dos_type.lower().replace('-', '_').replace('/', '_')
    r = context.dut[device].main().show_dos()
    assert r[0][dos_type].strip(
    ) == "Enable", f'dos setting fail: expect {dos_type}: Enable but {r[0][dos_type]}'


@when(
    u'{enable_disable} dos rule of {dos_type} on "{device}" created by previous step'
)
def step_impl(context, enable_disable, dos_type, device):
    dos_type = dos_type.lower().replace(' ', '-')
    if enable_disable.lower() == 'enable':
        context.dut[device].go_config().command(f'dos {dos_type}')
    else:
        context.dut[device].go_config().command(f'no dos {dos_type}')
    dos_type = dos_type.lower().replace('-', '_').replace('/', '_')
    r = context.dut[device].main().show_dos()
    assert r[0][dos_type].strip() == enable_disable.capitalize(
    ), f'dos setting fail: expect {dos_type}: {enable_disable.capitalize()} but {r[0][dos_type]}'


@given(u'{action} all dos rule on "{device}"')
def step_impl(context, action, device):
    dos_type_list = [
        'Null Scan', 'Xmas Scan', 'Nmap-ID Scan', 'SYN-FIN Scan', 'FIN Scan',
        'SYN-RST Scan', 'Nmap-Xmas Scan', 'New-TCP-without-SYN Scan'
    ]
    if issubclass(context.dut[device]._model, tn5000):
        dos_type_list[3] = 'SYN/FIN Scan'
        dos_type_list[5] = 'SYN/RST Scan'
        dos_type_list[7] = 'New-without-SYN Scan'

    if action == 'enable':
        for i in dos_type_list:
            dos_type = i.lower().replace(' ', '-')
            context.dut[device].go_config().command(f'dos {dos_type}')
    elif action == 'disable':
        for i in dos_type_list:
            dos_type = i.lower().replace(' ', '-')
            context.dut[device].go_config().command(f'no dos {dos_type}')
    else:
        raise ValueError(f'input invalid action: {action}')



from mdc.router.cli.rp_base.lg_router.edr800 import config
from mdc.router.cli.rp_base.lg_router.edr800.cli import PROMPT_CONFIG

class ConfigIfEthernet(config.Config):
    """
    Class for configuring Ethernet interfaces.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        # default apply the first element as parameter
        mod_port = self._args[0] if len(self._args) != 0 else None
        if mod_port is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'interface ethernet {mod_port}')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def set_switchport(self, vlan_id, mode="access"):
        ret = self._s.command_expect(f'switchport {mode} vlan {vlan_id}')
        return self

    def set_bridge_group(self, group: int = None):
        self._s.command_expect(f'bridge group {group}')
        return self

    def set_port_status(self, flag: str = 'enable'):
        """
        Set the port status.

            Args:
                flag: A parameter to decide port status

            Returns:
                self: The instance of the class.
        """
        if flag == 'enable':
            self._s.command_expect(f'no shutdown')
        elif flag == 'disable':
            self._s.command_expect(f'shutdown')
        else:
            raise ValueError(f'input invalid status: {flag}')
        return self



import os
import re
import json
import time

from datetime import datetime
from session import SshSession

client_config_file = '~/raddb/clients.conf'
users_config_file = '~/raddb/mods-config/files/authorize'

class RadiusService:
    """
    Class for autheticate via RADIUS server.

        Args:
            ssh_session (SshSession): SSH session object.
            sudo_paswd (str): Password for sudo command.
        
         Attributes:
            _ssh (SshSession): An SSH session object.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._sudo_pwd = sudo_paswd
        self._base_image = 'freeradius/freeradius-server'
        self._container_name = 'radius-server'


    def _check_conneted(self):
        if not self._ssh.is_connected():
            raise ValueError('SSH session not connected')


    def check_env(self):
        """
        Checks if the SSH session is connected and if the docker image exists.
            (Docker images version fix in v3.2)
            
        """
        self._check_conneted()
        check_images = f'docker images | grep {self._base_image} > check_images.log'
        self._ssh.command(check_images)
        output = self._ssh.command('cat check_images.log')
        if 'freeradius/freeradius-server' not in output:
            print(f'Ready to download image: {self._base_image}')
            self._ssh.command('docker pull freeradius/freeradius-server:latest-3.2', 
                              exact_prompts=['Downloaded newer image for freeradius/freeradius-server:latest-3.2'], 
                              timeout=120)


    def add_client(self,
                    client_name: str = None,
                    ipaddr: str = '192.168.127.0',
                    netmask: str = '24', 
                    secret: str = 'testing123', 
                    msg: str = 'no', 
                    nastype: str = 'other'):
        if client_name is None:
            raise ValueError(f'client_name is required.')
        
        client_config = f'client {client_name} {{\\n'
        client_config += f'\\tipaddr = {ipaddr}/{netmask}\\n'
        client_config += f'\\tproto = *\\n'
        client_config += f'\\tsecret = {secret}\n'
        client_config += f'\\trequire_message_authenticator = {msg}\\n'
        client_config += f'\\tnastype = {nastype}\\n}}'

        self._ssh.command(f'mkdir -p ~/raddb/mods-config/files')
        self._ssh.command(f'echo -e "{client_config}" >> {client_config_file}')

    
    def add_users(self,
                   username: str = None,
                   password: str = None,
                   service_type: str = 'Shell-User'):   # Shell-User = Admin; Login = User
        user_config = f'{username}\\tCleartext-Password := "{password}"\n'
        user_config += f'\\tService-Type = {service_type}\n'
        
        self._ssh.command(f'mkdir -p ~/raddb/mods-config/files')        
        self._ssh.command(f'echo -e \'{user_config}\' >> {users_config_file}')
        
        
    def build_local_image(self):
        dockerfile_config = f'FROM freeradius/freeradius-server:latest\\nCOPY raddb/ /etc/raddb/'
        self._ssh.command(f'echo -e "{dockerfile_config}" > ~/Dockerfile')
        
        # build image with custom config
        self._ssh.command(f'docker build -t {self._container_name}-image -f Dockerfile .')


    def start(self):
        """
        Starts custom freeradius container on the remote machine.

            Raises: 
                ValueError: If SSH session is not connected. 
        """
        self._check_conneted()
        cmd = f'docker run --rm -d --name {self._container_name} \
                -p 1812-1813:1812-1813/udp {self._container_name}-image'
        
        self._ssh.command(cmd)
        time.sleep(5)
        self._ssh.command('')  # wait prompt
    

    def stop(self):
        """
        Stop the container and clean up the environment.
             - stop container
             - remove custom freeradius image
             - remove radius local config file   
        """
        self._check_conneted()

        stop_cmd = f'docker stop {self._container_name}'
        self._ssh.command(stop_cmd)
        self._ssh.command('')  # wait prompt

        rmi_cmd = f'docker rmi {self._container_name}-image'
        self._ssh.command(rmi_cmd)

        rm_cmd = f'rm -r Dockerfile raddb'
        self._ssh.command(rm_cmd)



import re

from session import SshSession
from utils.decorator import root_access_required
from utils.input_chk import chk_valid_mac, chk_valid_ip

class ShellCmdService:
    """
    Class for using utility in Host.

        Args:
            ssh_session (SshSession): SSH session object.
            sudo_paswd (str): Password for sudo access.

        Attributes:
            _ssh (SshSession): SSH session object.
            _sudo_pwd (str): Password for sudo access.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._sudo_pwd = sudo_paswd

    def check_env(self):
        """
        Check if the host is alive.

            Raises:
                ValueError: If the host is not alive.
        """
        if not self._ssh:
            raise ValueError('host is not alive')

    @root_access_required
    def unzip_file(self, unzip_file, unzip_filepath, unzip_path):
        """
         The path to the file to unzip. Must be a gzipped tar file
         
         Args:
         	 unzip_filepath: The path to the unzip file
         	 unzip_path: The path to the unzip file
        """
        print(f'tar zxvf {unzip_filepath}/{unzip_file}.tar.gz -C {unzip_path}')
        ret = self._ssh.command(
            f'tar zxvf {unzip_filepath}/{unzip_file}.tar.gz -C {unzip_path}')

    @root_access_required
    def arp(self, count, ip):
        """
         Send ARP request to IP. This is a blocking call. The return value is a list of IP addresses that were sent to the IP.
         
            Args:
                count: Number of ARP requests to send. If this is less than 1 it is a zero - length list
                ip: IP address to send ARP request
            
            Returns: 
                A list of IP addresses that were sent to the
        """
        ret = self._ssh.command(f'arping -c {count} {ip}', timeout=5)
        return ret

    def tcpdump(self, itf, filter_type):
        """
         Get the PID of the tcpdump process. It is possible to filter by type but not by type
         
            Args:
                itf: ITERATOR or ITF name ( ex : i386_8 )
                filter_type: TCPDUMP_NONE or TCPDUMP_STANDARDEFF
            
            Returns: 
                PID of the tcpdump
        """
        self._ssh.login_as_root()
        ret = self._ssh.command(
            f'tcpdump -i {itf} {filter_type} > tcpdumpfile 2>&1 &')
        pid = re.findall("\[[\d\]]] \d+", ret)[0].split(' ')[1]
        return pid

    def get_tcpdumpfile(self, pid):
        """
         Get tcpdumpfile for process with pid. This is useful for debugging the process
         
            Args:
                pid: pid of the process to get tcpdumpfile for
            
            Returns: 
                string containing tcpdumpfile for the process with pid
        """
        self._ssh.command(f'kill {pid}')
        ret = self._ssh.command(f'cat tcpdumpfile')
        print(f"DUMP{ret}DUMPEND")
        self._ssh.command(f'rm tcpdumpfile')
        self._ssh.logout_from_root()
        return ret

    def dig(self, domain_name: str = None):
        """
        A tool to dig the URL.
            
            Args:
                domain_name: A domain name which is required to analyze
                
            Raises:
                ValueError: If the host is not alive.
        """
        ret = self._ssh.command(f'dig {domain_name} | grep {domain_name}.')
        return ret

    def show_network(self, dev=None):
        """
        Show network information.

            Args:
                dev (str, optional): The device name. Defaults to None.

            Returns:
                str: The network information. 
        """
        if dev is None:
            return self._ssh.command(f'ip addr')
        else:
            return self._ssh.command(f'ip addr show {dev}')
        
    @root_access_required
    def set_network(self, dev: str = 'eth1', ip: str = '', mask: str = '24'):
        """
        Set network for a device.

            Args:
                dev (str, optional): The device name. Defaults to 'eth1'.
                ip (str, optional): The IP address. Defaults to ''.
                mask (str, optional): The subnet mask. Defaults to '24'.
        """

        # Configure network setting via edit config file
        config=f"auto {dev}\niface {dev} inet static\naddress {ip}\nnetmask {mask}"

        self._ssh.command(f'ip addr flush {dev}')
        self._ssh.command('echo "source-directory /etc/network/interfaces.d" > /etc/network/interfaces')
        self._ssh.command(f'echo "{config}" > /etc/network/interfaces.d/{dev}')
        self._restart_network(dev)

    @root_access_required
    def restart_network(self, dev):
        self._restart_network(dev)
    
    def _restart_network(self, dev):
        """
        Restart network for a device.
        """
        self._ssh.command(f"ifdown {dev}")
        self._ssh.command(f"ifup {dev}")

    @root_access_required
    def set_networks(self, dev: str = 'eth1', ip: list = [], mask: list = []):
        """
        Set multiple networks for a device.

            Args: 
                dev (str, optional): The device name. Defaults to 'eth1'. 
                ip (list, optional): A list of IP addresses. Defaults to []. 
                mask (list, optional): A list of subnet masks. Defaults to []. 

            Raises: 
                ValueError: If the number of IPs and masks are not equal.  
        """
        if len(ip) != len(mask):
            raise ValueError('number of ip and mask are not equal')
        self._ssh.command(f'ip addr flush dev {dev}')

        config=f"auto {dev}\niface {dev} inet static\naddress {ip[0]}\nnetmask {mask[0]}"
        self._ssh.command('echo "source-directory /etc/network/interfaces.d" > /etc/network/interfaces')
        self._ssh.command(f'echo "{config}" > /etc/network/interfaces.d/{dev}')

        for i in range(1, len(ip)):
            self._ssh.command(f'echo "up ip addr add {ip[i]}/{mask[i]} dev {dev}" >> /etc/network/interfaces.d/{dev}')

    @root_access_required
    def set_mac_addr(self, dev: str = 'eth1', mac: str = ''):
        """
        Set the MAC address of a device.

            Args:
                dev (str): The device to set the MAC address for. Default is 'eth1'.
                mac (str): The MAC address to set. Must be a valid MAC address.

            Returns:
                str: The output of the command used to set the MAC address. 
        """
        chk_valid_mac(mac)
        ret = self._ssh.command(f'ip link set {dev} address {mac}')

    @root_access_required
    def remove_arp_entry(self, dev: str = 'eth1', ip: str = ''):
        """
        Remove an ARP entry from the device.

            Args:
                dev (str): The device name (default: 'eth1').
                ip (str): The IP address of the ARP entry to be removed.

            Returns: 
                str: The output of the command. 
        """
        chk_valid_ip(ip)
        ret = self._ssh.command(f'ip neigh del {ip} dev {dev}')

    @root_access_required
    def enable_dhcp_client(self, dev: str = 'eth1'):
        """
        Enable DHCP client on a given device.

            Args:
                dev (str): The device to enable DHCP client on. Defaults to 'eth1'.. Prints the output of the commands executed on the remote machine. 
        """
        print(self._ssh.command(f'ip addr flush {dev}'))
        print(self._ssh.command(f'dhclient {dev}', timeout=10))

    @root_access_required
    def add_route(self,
                  dev: str = 'eth1',
                  dip: str = '',
                  gw: str = '',
                  mask: str = ''):
        """
        Log in as root and add a route to the device. 

            Args: 
                dev (str): Name of the device. Default is 'eth1'. 
                dip (str): Destination IP address. 
                gw (str): Gateway IP address. 
                mask (str): Subnet mask. 
            Returns: 
                ret (list): List of routes on the device.  
        """

        # Configure routing rule via edit config file
        self._ssh.command(f'ip addr flush {dev}')
        self._ssh.command(f'echo "post-up ip route add {dip}/{mask} via {gw}" >> /etc/network/interfaces.d/{dev}')
        self._restart_network(dev)

    def show_route(self):
        """
        Show the route of the current device.

            Returns:
                str: The output of the command 'ip route'. 
        """
        return self._ssh.command('ip route')

    @root_access_required
    def remove_residual_artifact(self):
        self._ssh.command("rm *.log")


def parse_ping(content: str) -> dict:
    # SAMPLE OUTPUT
    # Ping to 1.1.1.1:
    # Packets: Sent = 4, Received = 4, Lost = 0
    idx_s = content.find('Sent = ')
    idx_r = content.find('Received = ')
    idx_l = content.find('Lost = ')
    if -1 in (idx_s, idx_r, idx_l):
        raise ValueError('Cannot parse ping content')
    return {
        'sent': int(content[idx_s:idx_r].split()[2].strip(', ')),
        'received': int(content[idx_r:idx_l].split()[2].strip(', ')),
        'lost': int(content[idx_l:].split()[2].strip())
    }


def parse_colon_fields(content: str) -> dict:
    """
    Input:
    content: str, string to parse
    ex:
    null-scan               : Disable
    xmas-scan               : Disable
    nmap-xmas-scan          : Disable
    syn/fin-scan            : Disable
    fin-scan                : Disable
    nmap-id-scan            : Disable
    syn/rst-scan            : Disable
    new-tcp-without-syn-scan: Disable
    Flash                   : Disable
    Syslog                  : Disable
    Trap                    : Disable

    Output:
    dict
    ex:
    {
        'null-scan':'Disable'
        'xmas-scan':'Disable'
        'nmap-xmas-scan':'Disable'
        'syn/fin-scan':'Disable'
        'fin-scan':'Disable'
        'nmap-id-scan':'Disable'
        'syn/rst-scan':'Disable'
        'new-tcp-without-syn-scan':'Disable'
        'Flash':'Disable'
        'Syslog':'Disable'
        'Trap':'Disable'
    }
    """
    ret = {}
    for line in content.split('\n'):
        try:
            k, v = line.split(':', 1)
        except ValueError:
            # if no char ':', split will raise
            # ValueError: not enough values to unpack (expected 2, got 1)
            continue
        ret[k.strip()] = v.strip()
    return ret


def parse_colon_fields_structure(content: str) -> dict:
    ret = {}
    lvs = [[ret, None]]  # levels information. list of [root, indent]
    pk, pv = None, None

    for line in content.split('\n'):
        parts = line.rstrip().split(':')
        if len(parts) != 2:
            continue
        # print('parts', parts)  # debug

        # get indent legth of current line
        _indent = len(parts[0]) - len(parts[0].lstrip())

        # get 1st level indent
        if lvs[0][1] is None:
            lvs[0][1] = _indent

        root, indent = lvs[-1]
        k = parts[0].strip()
        v = parts[1].strip()

        # print('root, indent, _indent', root, indent, _indent)  # debug
        # print('pk, pv', pk, pv)  # debug
        if _indent == indent:
            root[k] = v
        elif _indent < indent:
            # find upper level
            try:
                lvs.pop()
                root, indent = lvs[-1]
                while _indent != indent:
                    lvs.pop()
                    root, indent = lvs[-1]
            except IndexError as e:
                raise ValueError(
                    f'cannot align upper level at line: "{line}"') from e
            lvs[-1][0][k] = v
        else:  # _indent > indent
            if pv == '':
                new = {k: v}
                lvs[-1][0][pk] = new
                lvs.append([new, _indent])
            else:
                raise ValueError(
                    f'indent too long to align previous line: "{line}"')
        pk, pv = k, v

    return ret


def parse_show_ipsec(content: str) -> dict:

    def parse(line):
        ret = {}
        # support max line length is 200
        fields_align = ((0, 20), (22, 36), (37, 57), (59, 200))
        for i in range(0, len(fields_align), 2):
            kch, kct = fields_align[i]  # key column head/tail
            vch, vct = fields_align[i + 1]
            k = line[kch:kct].strip()
            v = line[vch:vct].strip()
            ret[k] = v
        return ret

    ret = {}
    cur = ret
    for line in content.split('\n'):
        colon_count = line.count(':')
        if colon_count == 2:
            cur |= parse(line)
        elif colon_count == 1:
            k, v = line.split(':')
            cur[k.strip()] = v.strip()
        else:
            section = line.strip()
            if len(section) == 0:
                continue  # skip empty line
            cur = {}
            ret[section] = cur
    return ret


def parse_dos_fields(content: str) -> dict:
    """
    Input:
    content: str, string to parse
    ex:
    null-scan               : Disable
    xmas-scan               : Disable
    nmap-xmas-scan          : Disable
    syn/fin-scan            : Disable
    fin-scan                : Disable
    icmp-death              : Disable    Limit: 1000(pkt/s)
    syn-flood               : Disable    Limit: 1000(pkt/s)
    arp-flood               : Disable    Limit: 1000(pkt/s)

    Output:
    dict
    ex:
    {
        'null-scan':'Disable',
        'xmas-scan':'Disable',
        'nmap-xmas-scan':'Disable',
        'syn/fin-scan':'Disable',
        'fin-scan':'Disable',
        'nmap-id-scan':'Disable',
        'syn/rst-scan':'Disable',
        'new-tcp-without-syn-scan':'Disable',
        'icmp-death':{'Status': 'Disable', 'Limit': '1000(pkt/s)'},
        'syn-flood':{'Status': 'Disable', 'Limit': '1000(pkt/s)'},
        'arp-flood':{'Status': 'Disable', 'Limit': '1000(pkt/s)'},
        'Severity':'<0> Emergency',
        'Flash':'Disable',
        'Syslog':'Disable',
        'Trap':'Disable',
    }
    """

    result = parse_colon_fields(content)
    for dos_name, value in result.items():
        if dos_name in ['icmp-death', 'syn-flood', 'arp-flood']:
            content = value.split(' ')
            status = content[0].strip()
            limit = content[-1].strip()
            result[dos_name] = {'Status': status, 'Limit': limit}

    return result


def parse_event_log(content: str) -> list:
    """
    parse the log event type of info.
    First, find the headers by searching for "--------------".
    Second, record the start index of each head.
    Third, parse the meaningful line with the indexes.
    Fourth, make each event as a dict and add to the ret.

    Input:
    content: str, the raw data

    Output:
    list of dict
        ex:
        [
            {'Index': '1', 'Date': '2022/06/09',
             'Time': '01:59:00',
             'Severity': '<0>',
             'Event': '[Auth Ok, Login Success]'},
            {'Index': '2',
             'Date': '2022/06/09',
             'Time': '01:56:07',
             'Severity': '<0>',
             'Event': '[Auth Ok, Login Success]'}
        ]
    """
    fields = ['Index', 'Date', 'Time', 'Severity', 'Event']
    result = []
    for line in content.splitlines():
        parts = line.split(None, 4)
        if len(parts) != 5 or parts[0].isdigit() is False:
            continue  # skip if bad format
        result.append(dict(zip(fields, parts)))
    return result


def parse_event_log_v2(content: str) -> list:
    ret = []
    delimiter = '-' * 76
    for block in content.split(delimiter):
        if len(block.strip()) == 0:
            continue  # TODO: improve format check
        ret.append(parse_dos_fields(block))
    return ret


def parse_table(content) -> list:

    def is_title_separator(line: str):
        chars = set()
        for c in line.strip():  # strip \r \n etc
            chars.add(c)
        return chars == {' ', '-'}

    def get_columns(separator):
        columns = []
        h = t = 0
        for i, c in enumerate(separator):
            t = i
            if c == ' ':
                columns.append((h, t))
                h = t + 1
        t += 1
        columns.append((h, t))
        return list(filter(lambda c: c[0] < c[1], columns))

    def parse(columns, line):
        items = []
        for c in columns:
            items.append(line[c[0]:c[1]].strip())
        return items

    fields = None
    columns = None
    result = []
    lines = content.split('\n')
    for i, line in enumerate(lines):
        if columns is None:
            if is_title_separator(line):
                columns = get_columns(line)
                fields = parse(columns, lines[i - 1])
        else:
            _line = line.rstrip()
            if len(_line) == 0:
                continue  # ignore empty line
            result.append(dict(zip(fields, parse(columns, _line))))
    return result



import glob
feature_files = glob.glob('features/*.feature')

def line_exists(line, filename):
    with open(filename) as f:
        return line in f.read()

with open('fail_test') as qq_file, open(feature_files[0], 'r+') as data_file, open('features/flaky.feature', 'w+') as flaky:
    qq_lines = qq_file.readlines()
    data_lines = data_file.readlines()
    for line in qq_lines:
        line = line.strip()
        # 忽略空行
        if not line:
            continue
        for i in range(len(data_lines)):
            if line in data_lines[i]:
                # print(type(data_lines[i-1]))
                tmp = data_lines[i-1].split(" ")
                tmp.insert(-1, "@flaky")
                data_lines[i-1] = " ".join(tmp)
                print(data_lines[i-1])
    for i in data_lines:
        flaky.write(i)

print("處理完成。已在 data 檔案的對應 Scenario 上方加上 @flaky")



from behave import *

@when('set IPSec tunnel on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config_ipsec(context.table[0]['vpn_name']).set_vpn_connection(
                                remote_gateway=context.table[0]["remote-gateway"], 
                                interface="WAN", 
                                startup_mode=context.table[0]["startup-mode"],
                                local_multi_network=context.table[0]["local-multi-network"], 
                                remote_multi_network=context.table[0]["remote-multi-network"],
                                dpd_action=context.table[0]["dpd-action"], 
                                dpd_delay=context.table[0]["dpd-delay"], 
                                dpd_timeout=context.table[0]["dpd-timeout"],
                                identity=context.table[0]["identity"])

    context.dut[device].go_config_ipsec(context.table[0]['vpn_name']).go_phase1().set_key_exchange(
                                ike_mode=context.table[0]["ike-mode"],
                                ike_version=context.table[0]["ike-version"],
                                auth_mode_psk=context.table[0]["auth-mode psk"],
                                encryption=context.table[0]["encryption"],
                                hash_algo="sha256",
                                dh_group="2048",
                                life_time=context.table[0]["life-time"])


    context.dut[device].go_config_ipsec(context.table[0]['vpn_name']).go_phase2().set_data_exchange(
                                pfs=context.table[0]["pfs"], 
                                encryption=context.table[0]["encryption"], 
                                hash_algo="sha256", 
                                life_time=context.table[0]["life-time"])


